@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,13 +85,12 @@ DllEntryPoint (HINSTANCE hInstance, DWORD seginfo,
 #pragma aux (__cdecl) Tcl_AppendResult
 #pragma aux (__cdecl) Tcl_SetResult
 #pragma aux (__cdecl) Tcl_CreateCommand
-#pragma aux (__cdecl) Tk_CreatePhotoImageFormat
 #endif
 
 /*
  * cvs version
  */
-char * uutcl_id = "$Id: uutcl.c,v 1.7 1996/07/11 20:31:46 fp Exp $";
+char * uutcl_id = "$Id: uutcl.c,v 1.14 2002/03/06 13:52:45 fp Exp $";
 
 /*
  * data for our Callbacks
@@ -137,29 +136,35 @@ SendAFile (Tcl_Interp *interp,
 	   FILE *infile,   char *infname,
 	   int encoding,   int linperfile,
 	   char *outfname, char *towhom,
-	   char *subject,  int isemail)
+	   char *subject,  char *from,
+	   char *replyto,  int isemail)
 {
   char *command, *rcptlist, *ptr;
   FILE *thepipe, *theifile;
-  int len, res, count, part;
+  int len, count, res, part;
 
   if (towhom==NULL ||
       (outfname==NULL&&infname==NULL) || (infile&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
-    interp->result = "oops: Parameter check failed in SendAFile()";
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED)) {
+    Tcl_SetResult (interp, "oops: Parameter check failed in SendAFile()",
+		   TCL_STATIC);
     return UURET_ILLVAL;
   }
 
 #ifndef HAVE_POPEN
-  interp->result = "error: Your system does not support sending of files";
+  Tcl_SetResult (interp, "error: Your system does not support sending of files",
+		 TCL_STATIC);
   return UURET_ILLVAL;
 #else
   if (isemail && (uue_mailprog == NULL || *uue_mailprog == '\0')) {
-    interp->result = "error: Cannot Email file: option not configured";
+    Tcl_SetResult (interp, "error: Cannot Email file: option not configured",
+		   TCL_STATIC);
     return UURET_ILLVAL;
   }
   else if (!isemail && (uue_inewsprog == NULL || *uue_inewsprog == '\0')) {
-    interp->result = "error: Cannot Post file: option not configured";
+    Tcl_SetResult (interp, "error: Cannot Post file: option not configured",
+		   TCL_STATIC);
     return UURET_ILLVAL;
   }
 
@@ -168,12 +173,14 @@ SendAFile (Tcl_Interp *interp,
       ((isemail)?0:strlen(towhom)) + 32;
 
   if ((command = (char *) malloc (len)) == NULL) {
-    interp->result = "error: Out of memory allocating some bytes";
+    Tcl_SetResult (interp, "error: Out of memory allocating some bytes",
+		   TCL_STATIC);
     return UURET_NOMEM;
   }
 
   if ((rcptlist = (char *) malloc (strlen (towhom) + 16)) == NULL) {
-    interp->result = "error: Out of memory allocating some bytes";
+    Tcl_SetResult (interp, "error: Out of memory allocating some bytes",
+		   TCL_STATIC);
     _FP_free (command);
     return UURET_NOMEM;
   }
@@ -219,6 +226,18 @@ SendAFile (Tcl_Interp *interp,
     }
   }
 
+  if (from && *from == '\0') {
+    from = NULL;
+  }
+
+  if (subject && *subject == '\0') {
+    subject = NULL;
+  }
+
+  if (replyto && *replyto == '\0') {
+    replyto = NULL;
+  }
+  
   /*
    * Get going ...
    */
@@ -252,9 +271,10 @@ SendAFile (Tcl_Interp *interp,
 #endif
     }
 
-    res = UUE_PrepPartial (thepipe, theifile, infname, encoding,
-			   outfname, 0, part, linperfile, 0,
-			   rcptlist, NULL, subject, isemail);
+    res = UUE_PrepPartialExt (thepipe, theifile, infname, encoding,
+			      outfname, 0, part, linperfile, 0,
+			      rcptlist, from, subject, replyto,
+			      isemail);
 
 #if 0
     if (UUGetOption (UUOPT_VERBOSE, NULL, NULL, 0)) {
@@ -336,6 +356,10 @@ uutcl_UpdateParameter (Tcl_Interp *interp)
     UUSetOption (UUOPT_USETEXT, atoi (cval), NULL);
   if ((cval = Tcl_GetVar (interp, "SaveFilePath",   TCL_GLOBAL_ONLY))!=NULL)
     UUSetOption (UUOPT_SAVEPATH, 0, cval);
+  if ((cval = Tcl_GetVar (interp, "OptionRemove",   TCL_GLOBAL_ONLY))!=NULL)
+    UUSetOption (UUOPT_REMOVE, atoi (cval), NULL);
+  if ((cval = Tcl_GetVar (interp, "OptionMoreMime", TCL_GLOBAL_ONLY))!=NULL)
+    UUSetOption (UUOPT_MOREMIME, atoi (cval), NULL);
 }
 
 /*
@@ -376,12 +400,6 @@ uutcl_Info (ClientData clientData, Tcl_Interp *interp,
     Tcl_AppendResult (interp, " { have_tk 1 } ", NULL);
 #else
     Tcl_AppendResult (interp, " { have_tk 0 } ", NULL);
-#endif
-  if (argc==1 || uutcl_HaveArg (argc, argv, "tk_photo"))
-#ifdef TK_PHOTO
-    Tcl_AppendResult (interp, " { tk_photo 1 } ", NULL);
-#else
-    Tcl_AppendResult (interp, " { tk_photo 0 } ", NULL);
 #endif
   if (argc==1 || uutcl_HaveArg (argc, argv, "have_mail")) {
 #ifdef PROG_MAILER
@@ -531,13 +549,14 @@ uutcl_GetListOfFiles (ClientData clientData, Tcl_Interp *interp,
   while ((iter=UUGetFileListItem(count))) {
     if (((iter->state & UUFILE_OK) || 
 	 UUGetOption (UUOPT_DESPERATE, NULL, NULL, 0)) && iter->filename) {
-      sprintf (tmpstring, " { %d %d %s %s %s {",
+      sprintf (tmpstring, " { %d %d {%s} %s %s {",
 	       count, iter->state, iter->filename,
 	       (iter->mimetype)?iter->mimetype:"{}",
 	       (iter->uudet == UU_ENCODED) ? "UUdata " :
 	       (iter->uudet == B64ENCODED) ? "Base64 " :
 	       (iter->uudet == XX_ENCODED) ? "XXdata " :
-	       (iter->uudet == BH_ENCODED) ? "Binhex " : "Text");
+	       (iter->uudet == BH_ENCODED) ? "Binhex " : 
+	       (iter->uudet == YENC_ENCODED) ? "yEnc" : "Text");
 
       if (iter->haveparts) {
 	sprintf (t2, "%s%s%d ",
@@ -625,7 +644,7 @@ uutcl_DecodeFile (ClientData clientData, Tcl_Interp *interp,
   }
 
   if ((iter = UUGetFileListItem (atoi (argv[1]))) == NULL) {
-    interp->result = "invalid file number";
+    Tcl_SetResult (interp, "invalid file number", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -662,7 +681,7 @@ uutcl_GetTempFile (ClientData clientData, Tcl_Interp *interp,
   }
 
   if ((iter = UUGetFileListItem (atoi (argv[1]))) == NULL) {
-    interp->result = "invalid file number";
+    Tcl_SetResult (interp, "invalid file number", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -678,7 +697,7 @@ uutcl_GetTempFile (ClientData clientData, Tcl_Interp *interp,
   }
 
   if (iter->binfile == NULL) {
-    interp->result = "unknown error while decoding";
+    Tcl_SetResult (interp, "unknown error while decoding", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -749,7 +768,7 @@ uutcl_InfoFile (ClientData clientData, Tcl_Interp *interp,
   }
 
   if ((iter = UUGetFileListItem (atoi (argv[1]))) == NULL) {
-    interp->result = "invalid file number";
+    Tcl_SetResult (interp, "invalid file number", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -802,7 +821,7 @@ uutcl_ListFile (ClientData clientData, Tcl_Interp *interp,
   }
 
   if ((iter = UUGetFileListItem (atoi (argv[1]))) == NULL) {
-    interp->result = "invalid file number";
+    Tcl_SetResult (interp, "invalid file number", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -822,7 +841,7 @@ uutcl_ListFile (ClientData clientData, Tcl_Interp *interp,
     return TCL_ERROR;
 
   if (iter->binfile==NULL || (inpfile=fopen (iter->binfile, "r"))==NULL) {
-    interp->result = "couldn't read file";
+    Tcl_SetResult (interp, "couldn't read file", TCL_STATIC);
     return TCL_ERROR;
   }
   if ((inpfile = fopen (iter->binfile, "r")) == NULL) {
@@ -883,12 +902,12 @@ uutcl_Rename (ClientData clientData, Tcl_Interp *interp,
   }
 
   if (*argv[2] == '\0') {
-    interp->result = "illegal file name";
+    Tcl_SetResult (interp, "illegal file name", TCL_STATIC);
     return TCL_ERROR;
   }
 
   if ((iter = UUGetFileListItem (atoi (argv[1]))) == NULL) {
-    interp->result = "invalid file number";
+    Tcl_SetResult (interp, "invalid file number", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -936,40 +955,45 @@ uutcl_GetEncodeParams (Tcl_Interp *interp,
     *linperfile = atoi (argv[argv2]);
 
     if (*linperfile != 0 && *linperfile < 200) {
-      interp->result = "illegal number of lines per file";
+      Tcl_SetResult (interp, "illegal number of lines per file", TCL_STATIC);
       return TCL_ERROR;
     }
   }
   if (argv1 && argv1 < argc) {
-    switch (atoi (argv[argv1])) {
-    case 0:
+    switch (*argv[argv1]) {
+    case '0':
+    case 'u':
+    case 'U':
       *encoding = UU_ENCODED;
       break;
-    case 1:
+    case '1':
+    case 'x':
+    case 'X':
       *encoding = XX_ENCODED;
       break;
-    case 2:
+    case '2':
+    case 'b':
+    case 'B':
       *encoding = B64ENCODED;
       break;
-    default:
-      switch (*argv[argv1]) {
-      case 'u':
-      case 'U':
-	*encoding = UU_ENCODED;
-	break;
-      case 'x':
-      case 'X':
-	*encoding = XX_ENCODED;
-	break;
-      case 'b':
-      case 'B':
-	*encoding = B64ENCODED;
-	break;
-      default:
-	interp->result = "invalid encoding method";
-	return TCL_ERROR;
-      }
+    case '3':
+    case 't':
+    case 'T':
+      *encoding = PT_ENCODED;
       break;
+    case '4':
+    case 'q':
+    case 'Q':
+      *encoding = QP_ENCODED;
+      break;
+    case '5':
+    case 'y':
+    case 'Y':
+      *encoding = YENC_ENCODED;
+      break;
+    default:
+      Tcl_SetResult (interp, "invalid encoding method", TCL_STATIC);
+      return TCL_ERROR;
     }
   }
   return TCL_OK;
@@ -978,7 +1002,10 @@ uutcl_GetEncodeParams (Tcl_Interp *interp,
 /*
  * Encode and store in a file.
  * Syntax:
- *    uu_EncodeToFile source path [ dest subject intro lines encoding ]
+ *    uu_EncodeToFile source path \
+ *                    [ dest subject intro lines encoding from replyto]
+ *
+ * Most arguments are just for compatibilty with the other encoding procs.
  */
 
 static int UUTCLFUNC
@@ -986,10 +1013,10 @@ uutcl_EncodeToFile (ClientData clientData, Tcl_Interp *interp,
 		    int argc, char *argv[])
 {
   int encoding=UU_ENCODED, linperfile=0, res;
-  char errstring[256], olddir[256], newdir[256];
+  char errstring[256], olddir[256];
 
-  if (argc < 3 || argc > 8) {
-    interp->result = "wrong # args";
+  if (argc < 3 || argc > 10) {
+    Tcl_SetResult (interp, "wrong # args", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -1003,15 +1030,6 @@ uutcl_EncodeToFile (ClientData clientData, Tcl_Interp *interp,
   UUGetOption (UUOPT_SAVEPATH, NULL, olddir, 256);
   UUSetOption (UUOPT_SAVEPATH, 0, argv[2]);
 
-#if 0
-  if ((res = UUEncodeFile (encoding, linperfile,
-			   argv[1], argv[2],          /* source, path */
-			   (argc>3) ? argv[3] : NULL, /* destname     */
-			   (argc>5) ? argv[5] : NULL, /* introfile    */
-			   (argc>4) ? argv[4] : NULL, /* subject      */
-			   UUE_TOFILE, NULL)) != UURET_OK);
-#endif
-
   if ((res = UUEncodeToFile (NULL, argv[1], encoding,
 			     (argc>3) ? argv[3] : NULL,
 			     (argc>2) ? argv[2] : NULL,
@@ -1031,7 +1049,8 @@ uutcl_EncodeToFile (ClientData clientData, Tcl_Interp *interp,
 /*
  * Encode and send by email
  * Syntax:
- *    uu_EncodeToMail source addr [ dest subject intro lines encoding ]
+ *    uu_EncodeToMail source addr \
+ *                    [ dest subject intro lines encoding from replyto ]
  *
  * addr can be a single address or a list of addresses
  */
@@ -1043,8 +1062,8 @@ uutcl_EncodeToMail (ClientData clientData, Tcl_Interp *interp,
   int encoding=UU_ENCODED, linperfile=0, res;
   char errstring[256];
 
-  if (argc < 3 || argc > 8) {
-    interp->result = "wrong # args";
+  if (argc < 3 || argc > 10) {
+    Tcl_SetResult (interp, "wrong # args", TCL_STATIC);
     return TCL_ERROR;
   }
   uutcl_UpdateParameter (interp);
@@ -1054,17 +1073,13 @@ uutcl_EncodeToMail (ClientData clientData, Tcl_Interp *interp,
 			     6, &linperfile) != TCL_OK)
     return TCL_ERROR;
 
-#if 0
-  if ((res = UUEncodeFile (encoding, linperfile,
-			   argv[1], argv[2],          /* source, addr */
-			   (argc>3) ? argv[3] : NULL, /* destname     */
-			   (argc>5) ? argv[5] : NULL, /* introfile    */
-			   (argc>4) ? argv[4] : NULL, /* subject      */
-			   UUE_MAILTO, NULL)) != UURET_OK);
-#endif
   if ((res = SendAFile (interp, NULL, argv[1], encoding, linperfile,
-			(argc>3) ? argv[3] : NULL, argv[2],
-			(argc>4) ? argv[4] : NULL, 1)) != UURET_OK) {
+			/* outfname */ (argc>3) ? argv[3] : NULL,
+			/* towhom   */ argv[2],
+			/* subject  */ (argc>4) ? argv[4] : NULL,
+			/* from     */ (argc>8) ? argv[8] : NULL,
+			/* replyto  */ (argc>9) ? argv[9] : NULL,
+			1)) != UURET_OK) {
     /*
      * If res==UURET_ILLVAL, SendAMail has already filled in the result
      */
@@ -1083,7 +1098,8 @@ uutcl_EncodeToMail (ClientData clientData, Tcl_Interp *interp,
 /*
  * Encode and post to the news
  * Syntax:
- *    uu_EncodeToNews source addr [ dest subject intro lines encoding ]
+ *    uu_EncodeToNews source addr \
+ *                    [ dest subject intro lines encoding from replyto ]
  *
  * addr can be a single newsgroup or a list of newsgroups
  */
@@ -1095,8 +1111,8 @@ uutcl_EncodeToNews (ClientData clientData, Tcl_Interp *interp,
   int encoding=UU_ENCODED, linperfile=0, res;
   char errstring[256];
 
-  if (argc < 3 || argc > 8) {
-    interp->result = "wrong # args";
+  if (argc < 3 || argc > 10) {
+    Tcl_SetResult (interp, "wrong # args", TCL_STATIC);
     return TCL_ERROR;
   }
   uutcl_UpdateParameter (interp);
@@ -1106,17 +1122,13 @@ uutcl_EncodeToNews (ClientData clientData, Tcl_Interp *interp,
 			     6, &linperfile) != TCL_OK)
     return TCL_ERROR;
   
-#if 0
-  if ((res = UUEncodeFile (encoding, linperfile,
-			   argv[1], argv[2],          /* source, group */
-			   (argc>3) ? argv[3] : NULL, /* destname      */
-			   (argc>5) ? argv[5] : NULL, /* introfile     */
-			   (argc>4) ? argv[4] : NULL, /* subject       */
-			   UUE_POSTTO, NULL)) != UURET_OK)
-#endif
   if ((res = SendAFile (interp, NULL, argv[1], encoding, linperfile,
-			(argc>3) ? argv[3] : NULL, argv[2],
-			(argc>4) ? argv[4] : NULL, 0)) != UURET_OK) {
+			/* outfname */ (argc>3) ? argv[3] : NULL,
+			/* towhom   */ argv[2],
+			/* subject  */ (argc>4) ? argv[4] : NULL,
+			/* from     */ (argc>8) ? argv[8] : NULL,
+			/* replyto  */ (argc>9) ? argv[9] : NULL,
+			0)) != UURET_OK) {
     /*
      * If res==UURET_ILLVAL, SendAMail has already filled in the result
      */
@@ -1141,9 +1153,6 @@ int UUTCLEXPORT UUTCLFUNC
 Uu_Init (Tcl_Interp *interp)
 {
   char tmp[32];
-#ifdef TK_PHOTO
-  extern int Gfp_Init (Tcl_Interp *);
-#endif
 
   /*
    * Check whether we are already initialized
@@ -1157,7 +1166,7 @@ Uu_Init (Tcl_Interp *interp)
    */
 
   if (UUInitialize () != UURET_OK) {
-    interp->result = "Error initializing decoding engine";
+    Tcl_SetResult (interp, "Error initializing decoding engine", TCL_STATIC);
     return TCL_ERROR;
   }
 
@@ -1220,13 +1229,6 @@ Uu_Init (Tcl_Interp *interp)
   if (Tcl_GetVar (interp, "OptionUsetext", TCL_GLOBAL_ONLY) == NULL)
     Tcl_SetVar (interp, "OptionUsetext", tmp, TCL_GLOBAL_ONLY);
 
-#ifdef TK_PHOTO
-  /*
-   * Initialize the GFP Image format
-   */
-  (void) Gfp_Init (interp);
-#endif
-
   return TCL_OK;
 }
 
