@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,38 +89,38 @@ typedef struct _extlist {
  * Operational Flags
  */
 
-int incext    = 0, stdinput = 0;
-int overwrite = 0, interact = 1;
-int quiet     = 0, decoall  = 0;
-int nobar     = 0;
+static int incext    = 0, stdinput = 0;
+static int overwrite = 0, interact = 1;
+static int quiet     = 0, decoall  = 0;
+static int nobar     = 0, autoren  = 0;
 
 /*
  * Global Variables
  */
 
-extlist *extensions = NULL;
-char savepath[512];
+static extlist *extensions = NULL;
+static char savepath[512];
 
 /*
  * some statistics
  */
 
-int decodedok     = 0;
-int decodedfailed = 0;
-int decodedmisp   = 0;
-int inputerrors   = 0;
-int inputfiles    = 0;
+static int decodedok     = 0;
+static int decodedfailed = 0;
+static int decodedmisp   = 0;
+static int inputerrors   = 0;
+static int inputfiles    = 0;
 
 /*
  * version string
  */
 
-char uuversion[256] = VERSION "pl" PATCH;
+static char uuversion[256] = VERSION "pl" PATCH;
 
 /*
  * cvs version
  */
-char * uudeview_id = "$Id: uudeview.c,v 1.13 1996/12/19 23:33:46 fp Exp $";
+char * uudeview_id = "$Id: uudeview.c,v 1.28 2003/04/12 23:33:55 fp Exp $";
 
 static int	addexts		_ANSI_ARGS_((char *));
 static int	work_file	_ANSI_ARGS_((char *));
@@ -280,7 +280,7 @@ getyn (char *theline, char def)
 
   if (interact && !feof (stdin)) {
     fflush (stdout);
-    if ((ptr = fgets ((theline)?theline:line, 255, stdin)) == NULL) {
+    if ((ptr = _FP_fgets ((theline)?theline:line, 255, stdin)) == NULL) {
       if (theline) *theline='\0';
       return def;
     }
@@ -331,6 +331,41 @@ exists (char *filename)
   return 1;
 }
 
+/*
+ * Find a new unique filename by adding a unique number to the file name.
+ * Some want the unique name at the end, while others prefer to keep it
+ * before the file extension. Support both.
+ * So if autoren is positive, append the unique number at the end. If it
+ * is negative, insert it before the first dot in the filename.
+ */
+
+int
+findAlternateName (char *originalName, char *newName, int autoren)
+{
+  char *p;
+  int i;
+
+  for (i=1; i<32768; i++) {
+    if (autoren > 0) {
+      sprintf (newName, "%s.%d", originalName, i);
+    }
+    else {
+      strcpy (newName, originalName);
+      if ((p = strchr (newName, '.')) != NULL) {
+	sprintf (p, "_%d%s", i, strchr (originalName, '.'));
+      }
+      else {
+	sprintf (newName, "%s_%d", originalName, i);
+      }
+    }
+    if (!exists (newName)) {
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
 /*
  * Add another extension we shall or shall not decode
  */
@@ -381,7 +416,7 @@ work_file (char *filename)
     return 0;
 
   if ((ptr = _FP_strrchr (filename, '.')) == NULL)
-    return 1;
+    return incext ? 0 : 1;
 
   ptr++;
 
@@ -480,24 +515,65 @@ proc_stdin (void)
 static void
 makeparams (char *comline)
 {
-  char *argv[32], *p1, *p2;
-  int argc=2;
+  char *p1, *argv[32];
+  int argc=2, quote=0;
 
   if (comline==NULL || *comline=='\0')
     return;
 
-  argv[1] = p1 = comline;
+  p1 = comline;
 
-  while ((p2 = strchr (p1, ' ')) != NULL && argc < 32) {
-    *p2++ = '\0';
+  while (*p1 == ' ' || *p1 == '\t') {
+    p1++;
+  }
 
-    while (*p2 == ' ')
-      p2++;
-    if (*p2 == '\0')
+  argv[1] = p1;
+
+  while (*p1 && argc < 31) {
+    switch (*p1) {
+    case ' ':
+    case '\t':
+      if (!quote) {
+	if ((*argv[argc-1] == '"' || *argv[argc-1] == '\'') &&
+	    p1-1 != argv[argc-1] &&
+	    *(p1-1) == *argv[argc-1]) {
+	  *(p1-1) = '\0';
+	  argv[argc-1] += 1;
+	}
+	*p1++ = '\0';
+	while (*p1 == ' ' || *p1 == '\t')
+	  p1++;
+	argv[argc++] = p1;
+      }
+      else {
+	p1++;
+      }
       break;
 
-    argv[argc++] = p1 = p2;
+    case '"':
+    case '\'':
+      if (!quote) {
+	quote = *p1++;
+      }
+      else if (quote == *p1++) {
+	quote = 0;
+      }
+      break;
+      
+    default:
+      p1++;
+    }
   }
+
+  if (!quote) {
+    if ((*argv[argc-1] == '"' || *argv[argc-1] == '\'') &&
+	p1-1 != argv[argc-1] &&
+	*(p1-1) == *argv[argc-1]) {
+      *(p1-1) = '\0';
+      argv[argc-1] += 1;
+    }
+  }
+
   work_comline (argc, argv);
 }
 
@@ -508,9 +584,8 @@ makeparams (char *comline)
 static void 
 loadparfile (char *filename)
 {
-  char line[256], *argv[32], *p1, *p2, c;
+  char line[256], c;
   FILE *pfile;
-  int argc;
 
   if ((pfile = fopen (filename+1, "r")) == NULL) {
     fprintf (stderr, "Couldn't load parameter file %s: %s (ignored)\n",
@@ -518,8 +593,6 @@ loadparfile (char *filename)
     return;
   }
 
-  argv[0] = NULL;
-
   while (!feof (pfile)) {
     if (_FP_fgets (line, 256, pfile) == NULL)
       break;
@@ -583,26 +656,82 @@ work_comline (int argc, char *argv[])
     }
     else switch (*(argv[number] + 1)) {
     case '\0':
+      interact = 0;
+      if (overwrite == 0) {
+	overwrite = 1;
+      }
       proc_stdin ();
       break;
+    case 'a':
+      autoren = (*argv[number] == '+') ? -1 : 1;
+      break;
+    case 'b':
+      if (argv[number][2] == '0' || argv[number][2] == '(')
+	UUSetOption (UUOPT_BRACKPOL, 0, NULL);
+      else if (argv[number][2] == '1' || argv[number][2] == '[')
+	UUSetOption (UUOPT_BRACKPOL, 1, NULL);
+      else if (argv[number][2] == '\0')
+	UUSetOption (UUOPT_BRACKPOL, 1, NULL);
+      /* '])' Emacs feels happier if I close these brackets */
+      break;
+    case 'c':
+      UUSetOption (UUOPT_REMOVE, 1, NULL);
+      break;
+    case 'd':
+      UUSetOption (UUOPT_DESPERATE, 1, NULL);
+      UUSetOption (UUOPT_TINYB64,   1, NULL);
+      break;
     case 'e':
       if (number+1 < argc && *argv[number+1] == '.') {
         incext = (*argv[number] == '+') ? 1 : 0;
         addexts (argv[++number]);
       }
       break;
+    case 'f':
+      UUSetOption (UUOPT_FAST, 1, NULL);
+      break;
+    case 'h':
+      usage (argv[0]);
+      exit (99);
+      break;
     case 'i':
       if (*argv[number] == '+' && stdinput) {
 	fprintf (stderr, "WARNING: cannot interact when reading from stdin\n");
       }
       else {
 	interact  = (*argv[number] == '+') ? 1 : 0;
-	overwrite = (*argv[number] == '-') ? 1 : overwrite;
+	if (overwrite == 0 && *argv[number] == '-') {
+	  overwrite = 1;
+	}
       }
       break;
-    case 'd':
-      UUSetOption (UUOPT_DESPERATE, 1, NULL);
-      UUSetOption (UUOPT_TINYB64,   1, NULL);
+    case 'm':
+      UUSetOption (UUOPT_IGNMODE, 1, NULL);
+      break;
+    case 'n':
+      nobar = 1;
+      break;
+    case 'o':
+      if (*argv[number] == '-') {
+	overwrite = 1;
+      }
+      else {
+	overwrite = -1;
+      }
+      break;
+    case 'p':
+      if (number+1 < argc) {
+        strcpy (savepath, argv[++number]);
+	if (strlen (savepath)) {
+	  if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
+	    strcat (savepath, DIRSEPARATOR);
+	}
+      }
+      break;
+    case 'q':
+      UUSetOption (UUOPT_VERBOSE, 0, NULL);
+      quiet = 1;
+      nobar = 1;
       break;
     case 'r':
       UUSetOption (UUOPT_IGNREPLY, 1, NULL);
@@ -615,50 +744,29 @@ work_comline (int argc, char *argv[])
       else
 	UUSetOption (UUOPT_DUMBNESS, 42, NULL);
       break;
-    case 'p':
-      if (number+1 < argc) {
-        strcpy (savepath, argv[++number]);
-	if (strlen (savepath)) {
-	  if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
-	    strcat (savepath, DIRSEPARATOR);
-	}
-      }
-      break;
-    case 'o':
-      overwrite = 1;
-      break;
-    case 'f':
-      UUSetOption (UUOPT_FAST, 1, NULL);
-      break;
-    case 'b':
-      if (argv[number][2] == '0' || argv[number][2] == '(')
-	UUSetOption (UUOPT_BRACKPOL, 0, NULL);
-      else if (argv[number][2] == '1' || argv[number][2] == '[')
-	UUSetOption (UUOPT_BRACKPOL, 1, NULL);
-      else if (argv[number][2] == '\0')
-	UUSetOption (UUOPT_BRACKPOL, 1, NULL);
-      /* '])' Emacs feels happier if I close these brackets */
-      break;
-    case 'q':
-      quiet = 1;
-      break;
-    case 'v':
-      UUSetOption (UUOPT_VERBOSE, 0, NULL);
-      break;
-    case 'm':
-      UUSetOption (UUOPT_IGNMODE, 1, NULL);
-      break;
     case 't':
       UUSetOption (UUOPT_USETEXT, 1, NULL);
       break;
-    case 'n':
-      nobar = 1;
+    case 'v':
+      UUSetOption (UUOPT_VERBOSE, 0, NULL);
       break;
     case 'V':
       fprintf (stdout, "uudeview %spl%s compiled on %s\n",
 	       VERSION, PATCH, __DATE__);
       exit (0);
       break;
+    case 'z':
+      if (argv[number][2] >= '0' && argv[number][2] <= '9') {
+	UUSetOption (UUOPT_MOREMIME, argv[number][2]-'0', NULL);
+      }
+      else if (argv[number][2] == 'z') {
+	UUSetOption (UUOPT_MOREMIME, 2, NULL);
+      }
+      else {
+	res = UUGetOption (UUOPT_MOREMIME, NULL, NULL, 0);
+	UUSetOption (UUOPT_MOREMIME, res+1, NULL);
+      }
+      break;
     default:
       usage (argv[0]);
       exit (99);
@@ -683,10 +791,8 @@ moreCB (void *param, char *string)
 {
   struct mCBparm *data = (struct mCBparm *) param;
 
-#if 0
   if (IsHeaderLine (string))
     return 0;
-#endif
 
   if (data->aline+2 >= data->lines) {
     data->aline = 0;
@@ -814,7 +920,8 @@ DumpFileList (void)
 	    (iter->uudet == UU_ENCODED) ? "UUdata" :
 	    (iter->uudet == B64ENCODED) ? "Base64" :
 	    (iter->uudet == XX_ENCODED) ? "XXdata" :
-	    (iter->uudet == BH_ENCODED) ? "Binhex" : "Text");
+	    (iter->uudet == BH_ENCODED) ? "Binhex" : 
+	    (iter->uudet == YENC_ENCODED) ? "yEnc" : "Text");
 
     if (iter->haveparts) {
       printf ("%s%d%s ",
@@ -847,8 +954,8 @@ DumpFileList (void)
 static int 
 process_one (uulist *iter)
 {
-  char targetname[512];
-  char line[256], command[256], tmp, *ptr1, *ptr2;
+  char targetname[1024], renamedname[1024];
+  char line[1024], command[256], tmp, *ptr1, *ptr2;
   int res, escflag;
   char *name;
 
@@ -860,19 +967,18 @@ process_one (uulist *iter)
       break;
     }
 
-    if ((!quiet || interact) && !decoall) {
+    if (interact && !decoall) {
       printf ("  %s %-15s is %s   [d] (?=help) ",
 	      filemode((int)iter->mode),
 	      (iter->filename) ? (iter->filename) : "",
 	      (iter->state&UUFILE_OK) ? "OK" : "in error (DESPERATE MODE)");
-    }
-    
-    if ((!quiet || interact) && !decoall)
       tmp = getyn (line, 'd');
+    }
     else {
       line[0] = '\0';
       tmp = 'd';
     }
+
     for (ptr1=line; *ptr1 && *ptr1 != tmp; ptr1++);
     if (*ptr1++==tmp) {
       while (isspace(*ptr1)) ptr1++;
@@ -894,19 +1000,16 @@ process_one (uulist *iter)
     else if (tmp == 'c') {
       printf ("\n\n-------------------------------------------------------------------------------\n\n");
       printf ("\tYou are now using UUDEVIEW, the uudecoder by Frank Pilhofer\n\n");
-      printf ("\tThe  program  is  distibuted  under  the  terms  of the GNU\n");
+      printf ("\tThe program  is  distributed  under  the  terms  of the GNU\n");
       printf ("\tGeneral Public License.  You should have received a copy of\n");
       printf ("\tthe GPL with the uudeview program. In particular this means\n");
       printf ("\tthat the program is distributed without any warranty.\n\n");
       printf ("\tIf you like uudeview, you are encouraged to send the author\n");
       printf ("\ta postcard (find  the address  in the  README  file), or at\n");
       printf ("\tleast an email.\n\n");
-      printf ("\tCheck out uudeview's WWW home page:\n");
-      printf ("\t\thttp://www.uni-frankfurt.de/~fp/uudeview/\n\n");
-      printf ("\tYou can always find  the most recent  version via anonymous\n");
-      printf ("\tftp on\n");
-      printf ("\t\tftp.uni-frankfurt.de:/pub/dist/frank\n\n");
-      printf ("\t            Frank Pilhofer (fp@informatik.uni-frankfurt.de)\n\n");
+      printf ("\tCheck out uudeview's home page:\n");
+      printf ("\t\thttp://www.fpx.de/fp/Software/UUDeview/\n\n");
+      printf ("\t            Frank Pilhofer (fp@fpx.de)\n\n");
       printf ("-------------------------------------------------------------------------------\n\n");
       continue;
     }
@@ -939,7 +1042,7 @@ process_one (uulist *iter)
     else if (tmp == 'r') {
       if (strlen(ptr1) <= 1) {
 	printf ("Enter new filename: "); fflush (stdout);
-	if (fgets (line, 250, stdin) == NULL) {
+	if (_FP_fgets (line, 250, stdin) == NULL) {
 	  printf ("\nERROR: Could not get filename: %s\n",
 		  strerror (errno));
 	  line[0] = '\0';
@@ -956,7 +1059,7 @@ process_one (uulist *iter)
     else if (tmp == 'p') {
       if (strlen(ptr1) <= 1) {
 	printf ("Enter new path: "); fflush (stdout);
-	if (fgets (line, 250, stdin) == NULL) {
+	if (_FP_fgets (line, 250, stdin) == NULL) {
 	  printf ("\nERROR: Could not get path: %s\n", strerror (errno));
 	  line[0] = '\0';
 	}
@@ -965,6 +1068,19 @@ process_one (uulist *iter)
       if (strlen (ptr1) > 1) {
 	if (ptr1[strlen(ptr1)-1] == '\n')
 	  ptr1[strlen(ptr1)-1] = '\0';
+	/* check that path exists */
+	if (access(ptr1, W_OK|X_OK) == 0) {
+	  struct stat finfo;
+	  if (stat (ptr1, &finfo) != 0) {
+	    printf ("\nERROR: Could not get path: %s\n",
+		    strerror (errno));
+	    break;
+	  }
+	  if (!S_ISDIR(finfo.st_mode)) {
+	    printf("\nERROR: `%s' is not a directory\n", ptr1);
+	    break;
+	  }
+	}
 	strcpy (savepath, ptr1);
 	if (strlen (savepath)) {
 	  if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
@@ -1006,6 +1122,7 @@ process_one (uulist *iter)
       /*
        * Set Target directory
        */
+    make_target:
       sprintf (targetname, "%s%s%s",
 	       savepath, 
 	       (strlen(savepath) &&
@@ -1016,84 +1133,98 @@ process_one (uulist *iter)
       /*
        * check whether this file exists
        */
-      res = ' ';
-      if (exists (targetname) && !overwrite && res != 'o') {
-	printf ("*** Target File %s exists!\n", targetname);
-	while (exists (targetname) && !overwrite && res != 'o') {
-	  printf ("*** (O)verwrite, Overwrite (A)ll, (R)ename, (N)ext, (P)ath [o] ");
-	  fflush (stdout);
-	  /*
-	   * Ask the user for her/his choice. If 'r' or 'p', and there
-	   * is more data on the line, use it as reply
-	   */
 
-	  res = getyn (line, 'o');
+    retryexcheck:
+      if (exists (targetname) && overwrite!=1 && autoren &&
+	  findAlternateName (targetname, renamedname, autoren)) {
+	printf ("*** Target File %s exists - autorenamed to %s\n",
+		targetname, renamedname);
+	strcpy (targetname, renamedname);
+      }
 
-	  for (ptr1=line; *ptr1 && *ptr1 != res; ptr1++);
-	  if (*ptr1++==res) {
-	    while (isspace(*ptr1)) ptr1++;
+      if (exists (targetname) && overwrite == -1) {
+	printf ("*** Target File %s exists! (file not decoded)\n",
+		targetname);
+	return 1;
+      }
+
+      if (exists (targetname) && !overwrite) {
+	printf ("*** Target File %s exists! Options:\n", targetname);
+	res = ' ';
+
+	printf ("\
+*** (O)verwrite, Overwrite (A)ll, Overwrite Non(e),\n\
+*** (R)ename, Au(t)o-Rename, (N)ext, (P)ath [o] ");
+	fflush (stdout);
+
+	/*
+	 * Ask the user for her/his choice. If 'r' or 'p', and there
+	 * is more data on the line, use it as reply
+	 */
+
+	res = getyn (line, stdinput ? 'n' : 'o');
+
+	for (ptr1=line; *ptr1 && tolower(*ptr1) != res; ptr1++);
+	if (*ptr1++==res) {
+	  while (isspace(*ptr1)) ptr1++;
+	}
+
+	switch (res) {
+	case 'n': return 1;
+	case 'a': overwrite=1; goto retryexcheck;
+	case 'o': goto noexcheck;
+	case 'e': overwrite=-1; return 1;
+	case 't': autoren=1; goto retryexcheck;
+	case 'r': 
+	  if (strlen(ptr1) <= 1) {
+	    printf ("Enter new filename: "); fflush (stdout);
+	    if (_FP_fgets (line, 250, stdin) == NULL) {
+	      printf ("\nERROR: Could not get filename: %s\n",
+		      strerror (errno));
+	      line[0] = '\0';
+	    }
+	    ptr1 = line;
 	  }
-
-	  switch (res) {
-	  case 'n': return 1;
-	  case 'a': overwrite=1; break;
-	  case 'o': break;
-	  case 'r': 
-	    if (strlen(ptr1) <= 1) {
-	      printf ("Enter new filename: "); fflush (stdout);
-	      if (fgets (line, 250, stdin) == NULL) {
-		printf ("\nERROR: Could not get filename: %s\n",
-			strerror (errno));
-		line[0] = '\0';
-	      }
-	      ptr1 = line;
-	    }
-	    if (strlen (ptr1) > 1) {
-	      if (ptr1[strlen(ptr1)-1] == '\n')
-		ptr1[strlen(ptr1)-1] = '\0';
-	      UURenameFile (iter, ptr1);
-	    }
-	    if ((name = UUFNameFilter (iter->filename)) == NULL) {
-	      fprintf (stderr, "ERROR: couldn't get filename of %s (%s)\n",
-		       (iter->filename)?iter->filename:"(null)",
-		       (iter->subfname)?iter->subfname:"(null)");
-	      return 1;
-	    }
-	    break;
-	  case 'p':
-	    if (strlen(ptr1) <= 1) {
-	      printf ("Enter new path: "); fflush (stdout);
-	      if (fgets (line, 250, stdin) == NULL) {
-		printf ("\nERROR: Could not get path: %s\n",
-			strerror (errno));
-		line[0] = '\0';
-	      }
-	      ptr1 = line;
-	    }
-	    if (strlen (ptr1) > 1) {
-	      if (ptr1[strlen(ptr1)-1] == '\n')
-		ptr1[strlen(ptr1)-1] = '\0';
-	      strcpy (savepath, ptr1);
-	      if (strlen(savepath)) {
-		if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
-		  strcat (savepath, DIRSEPARATOR);
-	      }
-	    }
-	    break;
+	  if (strlen (ptr1) > 1) {
+	    if (ptr1[strlen(ptr1)-1] == '\n')
+	      ptr1[strlen(ptr1)-1] = '\0';
+	    UURenameFile (iter, ptr1);
 	  }
-	  if (res == 'r' || res == 'p') {
-	    sprintf (targetname, "%s%s%s",
-		     savepath,
-		     (strlen (savepath) &&
-		      savepath[strlen(savepath)-1]==DIRSEPARATOR[0]) ?
-		     "":DIRSEPARATOR,
-		     name);
-	    if (exists (targetname) && !overwrite) {
-	      printf ("*** Target File %s exists!\n", targetname);
+	  if ((name = UUFNameFilter (iter->filename)) == NULL) {
+	    fprintf (stderr, "ERROR: couldn't get filename of %s (%s)\n",
+		     (iter->filename)?iter->filename:"(null)",
+		     (iter->subfname)?iter->subfname:"(null)");
+	    return 1;
+	  }
+	  goto make_target;
+
+	case 'p':
+	  if (strlen(ptr1) <= 1) {
+	    printf ("Enter new path: "); fflush (stdout);
+	    if (_FP_fgets (line, 250, stdin) == NULL) {
+	      printf ("\nERROR: Could not get path: %s\n",
+		      strerror (errno));
+	      line[0] = '\0';
+	    }
+	    ptr1 = line;
+	  }
+	  if (strlen (ptr1) > 1) {
+	    if (ptr1[strlen(ptr1)-1] == '\n')
+	      ptr1[strlen(ptr1)-1] = '\0';
+	    strcpy (savepath, ptr1);
+	    if (strlen(savepath)) {
+	      if (savepath[strlen(savepath)-1]!=DIRSEPARATOR[0])
+		strcat (savepath, DIRSEPARATOR);
 	    }
 	  }
+	  goto make_target;
+
+	default:
+	  goto retryexcheck;
 	}
       }
+
+    noexcheck:
       if ((res = UUDecodeFile (iter, targetname)) != UURET_OK) {
 	if (UUISATTY(stderr) && !nobar)
 	  fprintf (stderr, "%70s\r", ""); /* clear progress information */
@@ -1123,7 +1254,7 @@ process_one (uulist *iter)
 #ifndef SYSTEM_QUICKWIN
     else if (tmp == 'e') {
       printf ("Enter command line ($) for file: "); fflush (stdout);
-      if (fgets (line, 256, stdin) == NULL)
+      if (_FP_fgets (line, 256, stdin) == NULL)
 	printf ("\nERROR: Could not get Command line: %s\n",
 		strerror (errno));
       else if (strlen (line) > 1) {
@@ -1257,14 +1388,19 @@ usage (char *argv0)
 	  VERSION, (PATCH[0]>'0')?"pl":"", (PATCH[0]>'0')?PATCH:"");
   printf ("  usage:\n");
   printf ("    uudeview [options] [file ...]\n\n");
-  printf ("\t-i\tDisable interactivity (do not ask, decode everything)\n");
-  printf ("\t-m\tIgnore the file mode of uuencoded files\n");
-  printf ("\t+e/-e\tInclude or exclude extensions exclusively\n");
-  printf ("\t-d\tSets 'desperate' mode (process incomplete files)\n");
-  printf ("\t-f\tFast mode. Only if each file holds no more than one part\n");
-  printf ("\t-o\tOK to overwrite files already here (default is to ask)\n");
-  printf ("\t-b1\tSelect alternate bracket policy\n");
-  printf ("\t-p <path>\tSets path where to save decoded binaries to\n\n");
+  printf ("  Options:\n");
+  printf ("\t-i      Disable interactivity (do not ask, decode everything)\n");
+  printf ("\t-a      Autorename (rename file if it exists on disk)\n");
+  printf ("\t-m      Ignore the file mode of uuencoded files\n");
+  printf ("\t+e/-e   Include or exclude extensions exclusively\n");
+  printf ("\t-d      Sets 'desperate' mode (process incomplete files)\n");
+  printf ("\t-f      Fast mode. Only if each file holds no more than one part\n");
+  printf ("\t-o/+o   -o OK to overwrite files, +o never overwrite files\n");
+  printf ("\t-b1     Select alternate bracket policy\n");
+  printf ("\t-p path Sets path where to save decoded binaries to\n");
+  printf ("\t-c      Autoclear files that were successfully decoded.\n");
+  printf ("\t-s      Be more strict on MIME adherance.\n");
+  printf ("\t-q      Quiet. Do not emit progress messages.\n\n");
 #if defined(SYSTEM_DOS) || defined(SYSTEM_QUICKWIN)
   printf ("  See Manual for more details\n\n");
 #else
@@ -1286,7 +1422,7 @@ main (int argc, char *argv[])
   int res;
 #ifdef SYSTEM_QUICKWIN
   struct _wsizeinfo ws;
-#endif    
+#endif
 
   /*
    * No Signal handler in QuickWin
@@ -1300,7 +1436,7 @@ main (int argc, char *argv[])
 #ifdef SYSTEM_QUICKWIN
   ws._version = _QWINVER;
   ws._type    = _WINSIZEMAX;
-  (void) _wabout        ("UUdeview for Windows\n(c) 1995 Frank Pilhofer\nfp@informatik.uni-frankfurt.de");
+  (void) _wabout        ("UUdeview for Windows\n(c) 1995 Frank Pilhofer\nfp@fpx.de");
   (void) _wsetscreenbuf (fileno(stdout), 16384);
   (void) _wsetscreenbuf (fileno(stderr), 16384);
   (void) _wsetsize      (fileno(stdout), &ws);
@@ -1340,22 +1476,25 @@ main (int argc, char *argv[])
     return 99;
   }
 
-  /*
-   * use options from UUDEVIEW environment variable
-   */
-  if ((getenv ("UUDEVIEW")) != NULL) {
-    makeparams (getenv ("UUDEVIEW"));
-  }
-
   /*
    * Setup Callback
    */
+
   UUSetMsgCallback (NULL, MessageCallback);
 
   if (UUInitialize () != UURET_OK) {
     fprintf (stderr, "oops: could not initialize decoding library\n");
     return 99;
   }
+
+  /*
+   * use options from UUDEVIEW environment variable
+   */
+
+  if ((getenv ("UUDEVIEW")) != NULL) {
+    makeparams (getenv ("UUDEVIEW"));
+  }
+
   if (argc < 2) {
     /*
      * can only be in uudecode compatibility mode
@@ -1420,7 +1559,7 @@ Select File-Exit to close window\n\
 	    inputfiles, (inputfiles==1)?"":"s",
 	    decodedfailed+decodedmisp);
   }
-  
+
   /*
    * determine return value
    */
