@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@
 #include <uufnflt.h>
 
 
-char * uuenview_id = "$Id: uuenview.c,v 1.14 1996/09/26 17:32:20 fp Exp $";
+char * uuenview_id = "$Id: uuenview.c,v 1.22 2002/03/06 13:52:46 fp Exp $";
 
 /*
  * mail and news software
@@ -78,6 +78,7 @@ int     uue_mpsubject = 0;
  * defines
  */
 
+#define UUE_STDOUT      0
 #define UUE_TOFILE	1
 #define UUE_MAILTO	2
 #define UUE_POSTTO	3
@@ -137,7 +138,7 @@ BusyCallback (void *param, uuprogress *progress)
 		 progress->numparts);
     for (count=0, stuff[25]='\0'; count<25; count++)
       stuff[count] = (count<pcts/4)?'#':'.';
-    fprintf (stderr, "encoding part%3d of%3d %s\r",
+    fprintf (stderr, "encoding part %3d/%3d %s\r",
 	     progress->partno, progress->numparts,
 	     stuff);
     fflush  (stderr);
@@ -156,9 +157,8 @@ usage (char *argv0)
     printf ("\n\
   uuencode -- a simple encoder (w) 1995 Frank Pilhofer\n\n\
   usage:\n\
-    %s [infile] remotefile\n\n\
-",
-	    argv0);
+    uuencode [infile] remotefile\n\n\
+");
     printf ("\
 \tinfile      the local file, where to read data from. If no infile\n\
 \t            is given, or infile is a single hyphen, the standard\n\
@@ -174,76 +174,36 @@ usage (char *argv0)
   printf ("\n\
   UUENVIEW %spl%s -- a simple encoder (w) 1995 Frank Pilhofer\n\n\
   usage:\n\
-    %s [-type] [-lines] ", VERSION, PATCH, argv0);
-#ifndef SYSTEM_DOS
-  printf ("[-m address] ");
-  printf ("[-p newsgroup] ");
-  printf ("\n\t\t");
-  printf ("[-s subject] ");
-  printf ("[-o[d path]] ");
-#else
-  printf ("[-od path] ");
+    uuenview [-type] [options] file [...]\n\n", VERSION, PATCH);
+  printf ("  Options:\n");
+  printf ("\t-b,-u,-x,-y Encode files as Base64, Uuencoding, Xx, yEnc,\n");
+  printf ("\t-t,-q       Plain text or Quoted-Printable, respectively\n");
+  printf ("\t-o          Encode to file(s) using the original's base name\n");
+  printf ("\t-od path    Same, but write files to selected path\n");
+#if defined(HAVE_POPEN) && defined(PROG_MAILER)
+  printf ("\t-m addr     Send file(s) by mail\n");
 #endif
-  printf ("[-a] ");
-  printf ("file(s)\n\n");
-  printf ("\
-\ttype      one of 'u', 'x', 'b' for UUencoding, XXencoding or Base64\n\
-\t          encoding, respectively. Defaults to uuencoding.\n\
-\tlines     sets the number of encoded lines per file. Values smaller\n\
-\t          than 200 are ignored. 1000 lines encode 45kb UU/XX and 57kb\n\
-\t          Base64 data. Default is unlimited.\n\
-");
-#ifndef SYSTEM_DOS
-  printf ("\
-\taddress   comma-separated list of email recepients.\n\
-\tnewsgroup comma-separated list of newsgroups to post the file to.\n\
-\tsubject   used on the subject line for emailing and posting.\n\
-");
+#if defined(HAVE_POPEN) && defined(PROG_INEWS)
+  printf ("\t-p group    Post file(s) to newsgroups\n");
 #endif
-  printf ("\
-\tfile(s)   one or more files to be encoded.\n\
-");
-#ifdef SYSTEM_DOS
-  printf ("\
-\t          The encoded files are saved to files with the same base\n\
-\t          name as the original but with an extension of .001, .002\n\
-\t          etc. If you give a pathname with the -od parameter, the\n\
-\t          files are written to that directory, otherwise the're\n\
-\t          written to the current working directory.\n\
-");
+  printf ("\t-a          Read message from stdin and attach files\n");
+  printf ("\t-lines      Chunk into messages of lines lines each\n");
+  printf ("\n");
+#if defined(SYSTEM_DOS) || defined(SYSTEM_QUICKWIN)
+  printf ("  See Manual for more details\n\n");
 #else
-  printf ("\
-\t          If -o is used, the encoded files will be saved to files\n\
-\t          with the same base name as the original but an extension\n\
-\t          of .001, .002 etc. If you use -od and give an existing\n\
-\t          pathname, the encoded files are written to that directory\n\
-\t          instead of the current working directory.\n\
-");
-  printf ("\
-\t          If neither of -o, -m, -p is specified, output is sent to\n\
-\t          standard output, and the maximum line count is ignored.\n\
-");
+  printf ("  See uuenview(1) for more details.\n\n");
 #endif
-  printf ("\n\
-  example:\n\
-");
-#ifdef SYSTEM_DOS
-  printf ("\
-    %s -b -2000 -o uuenview.exe\n\
-\tEncodes 'uuenview.exe' as Base64 into 2000-line chunks, and writes\n\
-\tthe result to uuenview.001 and uuenview.002.\n\
-\n\
-",
-	  argv0);
+  printf ("  Example:\n");
+#ifndef HAVE_POPEN
+  printf ("    uuenview -b -2000 -o uuenview.exe\n");
+  printf ("\tEncodes uuenview.exe as Base64 into 2000-line chunks, and\n");
+  printf ("\twrites the result to uuenview.001 and uuenview.002.\n\n");
 #else
-  printf ("\
-    %s -b -2000 -m root -o uudeview.tar.gz\n\
-\tEncodes 'uudeview.tar.gz' as Base64 into 2000-line chunks. It is\n\
-\tboth mailed to your system administrator and written to the files\n\
-\tuudeview.001 and uudeview.002\n\
-\n\
-",
-	  argv0);
+  printf ("    uuenview -b -2000 -m root -o uudeview.tar.gz\n");
+  printf ("\tEncodes 'uudeview.tar.gz' as Base64 into 2000-line chunks.\n");
+  printf ("\tIt is both mailed to your system administrator and written\n");
+  printf ("\tto the files uudeview.001 and uudeview.002\n\n");
 #endif
 
   return;
@@ -372,8 +332,9 @@ SendMkCommand (char **rcptlist, char *towhom, char *subject, int isemail)
     ptr = _FP_strtok (towhom, ",; ");
 
     while (ptr) {
-      strcat (command, " ");
+      strcat (command, " \"");
       strcat (command, ptr);
+      strcat (command, "\"");
 
       if (count++)
 	strcat (*rcptlist, ",");
@@ -409,7 +370,9 @@ SendMkCommand (char **rcptlist, char *towhom, char *subject, int isemail)
  */
 
 static int
-AttachFiles (char *towhom, char *subject, int isemail, int encoding,
+AttachFiles (char *towhom, char *subject,
+	     char *from, char *replyto,
+	     int isemail, int encoding,
 	     int argc, char *argv[])
 {
   static char input[1024], *ctype=NULL, *cte=NULL, boundary[64];
@@ -418,6 +381,7 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
   int index, res;
   /* remember the headers we care about */
   int hadsubject=0, hadgroups=0, hadto=0, hadmime=0, hadmulti=0;
+  int hadfrom=0, hadreplyto=0;
 
   if (towhom) {
 #ifndef HAVE_POPEN
@@ -455,6 +419,7 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
     /*
      * If the first line does not appear to be a header
      */
+
     if (index == 0) {
       ptr = input;
       while (*ptr && !isspace(*ptr) && *ptr!=':')
@@ -492,6 +457,24 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
       }
       hadto = 1;
     }
+    else if (_FP_strnicmp (input, "From:", 5) == 0) {
+      if (from) {
+	fprintf (thepipe, "From: %s\n", from);
+      }
+      else {
+	fprintf (thepipe, "%s", input);
+      }
+      hadfrom = 1;
+    }
+    else if (_FP_strnicmp (input, "Reply-To:", 9) == 0) {
+      if (replyto) {
+	fprintf (thepipe, "Reply-To: %s\n", replyto);
+      }
+      else {
+	fprintf (thepipe, "%s", input);
+      }
+      hadreplyto = 1;
+    }
     else if (_FP_strnicmp (input, "Mime-Version:", 13) == 0) {
       fprintf (thepipe, "%s", input);
       hadmime = 1;
@@ -529,40 +512,61 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
   /*
    * okay, header is copied. add our own fields if necessary
    */
-  if (!hadmime)
+
+  if (!hadmime && encoding != YENC_ENCODED)
     fprintf (thepipe, "Mime-Version: 1.0\n");
-  if (!hadmulti) {
+
+  if (!hadmulti && encoding != YENC_ENCODED) {
     /* must invent a boundary */
     sprintf (boundary, "==UUD_=_%ld", (long) time (NULL));
     fprintf (thepipe, "Content-Type: multipart/mixed; boundary=\"%s\"\n",
 	     boundary);
   }
 
-  if (index == 0) {
-    /*
-     * huh, there have been no headers.
-     */
-    if (subject) {
-      fprintf (thepipe, "Subject: %s\n", subject);
-    }
-    if (towhom && !isemail && rcptlist) {
-      fprintf (thepipe, "Newsgroups: %s\n", rcptlist);
-    }
-    if (towhom && isemail && rcptlist) {
-      fprintf (thepipe, "To: %s\n", rcptlist);
-    }
+  /*
+   * huh, there have been no headers.
+   */
+
+  if (!hadfrom && from) {
+    fprintf (thepipe, "From: %s\n", from);
   }
 
+  if (!hadgroups && towhom && !isemail && rcptlist) {
+    fprintf (thepipe, "Newsgroups: %s\n", rcptlist);
+  }
+
+  if (!hadto && towhom && isemail && rcptlist) {
+    fprintf (thepipe, "To: %s\n", rcptlist);
+  }
+
+  if (!hadreplyto && replyto) {
+    fprintf (thepipe, "Reply-To: %s\n", replyto);
+  }
+
+  if (!hadsubject && subject) {
+    fprintf (thepipe, "Subject: %s\n", subject);
+  }
+
+
   /*
    * end headers
    */
+
   fprintf (thepipe, "\n");
 
   /*
    * okay, copy text if available
    */
-  if (!feof (stdin)) {
-    if (!hadmulti) {
+
+  if (!feof (stdin) && index > 0) {
+    ptr = _FP_fgets (input, 1024, stdin);
+  }
+  else {
+    ptr = input;
+  }
+
+  if (!feof (stdin) && ptr) {
+    if (!hadmulti && encoding != YENC_ENCODED) {
       /*
        * need our own header
        */
@@ -581,22 +585,22 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
       }
       fprintf (thepipe, "\n");
 
-      if (index == 0) {
-	fprintf (thepipe, "%s", input);
-      }
-
       /*
        * just copy stdin
        */
 
+      fprintf (thepipe, "%s", input);
+
       while (!feof (stdin)) {
 	if (_FP_fgets (input, 256, stdin) == NULL)
 	  break;
 	fprintf (thepipe, "%s", input);
       }
+
       /*
        * the last crlf is to be ignored, so add another one
        */
+
       fprintf (thepipe, "\n");
     }
     else {
@@ -628,6 +632,8 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
       case 'p':
       case 's':
       case 'i':
+      case 'f':
+      case 'r':
 	if (index+1 < argc && argv[index+1][0] != '-')
 	  index++;
 	break;
@@ -635,6 +641,12 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
 	if (argv[index][2]=='d' && index+1<argc && argv[index+1][0]!='-')
 	  index++;
 	break;
+      case 'b': encoding = B64ENCODED; break;
+      case 'u': encoding = UU_ENCODED; break;
+      case 'x': encoding = XX_ENCODED; break;
+      case 't': encoding = PT_ENCODED; break;
+      case 'q': encoding = QP_ENCODED; break;
+      case 'y': encoding = YENC_ENCODED; break;
       default:  /* parameter without option */
 	break;
       }
@@ -643,7 +655,10 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
       /*
        * new boundary
        */
-      fprintf (thepipe, "--%s\n", boundary);
+
+      if (encoding != YENC_ENCODED) {
+	fprintf (thepipe, "--%s\n", boundary);
+      }
 
       if ((res = UUEncodeMulti (thepipe, NULL, argv[index], encoding,
 				NULL, NULL, 0)) != UURET_OK) {
@@ -655,10 +670,14 @@ AttachFiles (char *towhom, char *subject, int isemail, int encoding,
       fprintf (thepipe, "\n");
     }
   }
+
   /*
    * done.
    */
-  fprintf (thepipe, "--%s--\n\n", boundary);
+
+  if (encoding != YENC_ENCODED) {
+    fprintf (thepipe, "--%s--\n\n", boundary);
+  }
 
   if (towhom) {
 #ifdef HAVE_POPEN
@@ -682,7 +701,8 @@ static int
 SendAFile (FILE *infile,   char *infname,
 	   int encoding,   int linperfile,
 	   char *outfname, char *towhom,
-	   char *subject,  int isemail)
+	   char *subject,  char *from,
+	   char *replyto,  int isemail)
 {
   char *command, *rcptlist;
   FILE *thepipe, *theifile;
@@ -690,7 +710,8 @@ SendAFile (FILE *infile,   char *infname,
 
   if (towhom==NULL ||
       (outfname==NULL&&infname==NULL) || (infile==NULL&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     fprintf (stderr, "oops: Parameter check failed in SendAFile()\n");
     return UURET_ILLVAL;
   }
@@ -741,9 +762,10 @@ SendAFile (FILE *infile,   char *infname,
     }
 #endif
 
-    res = UUE_PrepPartial (thepipe, theifile, infname, encoding,
-			   outfname, 0, part, linperfile, 0,
-			   rcptlist, NULL, subject, isemail);
+    res = UUE_PrepPartialExt (thepipe, theifile, infname, encoding,
+			      outfname, 0, part, linperfile, 0,
+			      rcptlist, from, subject, replyto,
+			      isemail);
 
 #if 0
     if (UUGetOption (UUOPT_VERBOSE, NULL, NULL, 0)) {
@@ -779,9 +801,12 @@ int
 main (int argc, char *argv[])
 {
   int outflags[5], tostdout, linperfile, uuencode, encoding;
-  int index, introfile, subject, count, fileflag, stdinused;
+  int index, subject, count, fileflag, stdinused;
+  int from, replyto;
   char filename[512], usename[512], outdir[512];
-  int iskipflag, attach;
+  int iskipflag, attach, files;
+  char *p1, *myargv[256];
+  int myargc, quote;
   FILE *testit;
 
   /*
@@ -791,19 +816,22 @@ main (int argc, char *argv[])
   outflags[UUE_MAILTO] = -1;
   outflags[UUE_POSTTO] = -1;
   subject              = -1;
-  introfile            = -1;
   tostdout             =  1;
   linperfile           =  0;
   uuencode             =  0;
-  encoding             =  UU_ENCODED;
+  encoding             =  -1;
   count                =  0;
   stdinused            =  0;
   attach               =  0;
+  files                =  0;
+  from                 = -1;
+  replyto              = -1;
 
   if (UUInitialize () != UURET_OK) {
     fprintf (stderr, "oops: could not initialize decoding library\n");
     return 2;
   }
+
   UUSetOption (UUOPT_VERBOSE, 0, NULL);
 
 #if defined(SYSTEM_DOS) || defined(SYSTEM_QUICKWIN)
@@ -827,8 +855,6 @@ main (int argc, char *argv[])
     usage (argv[0]);
     return 1;
   }
-  if (_FP_stristr (argv[0], "uuencode") != NULL)
-    uuencode = 1; /* uuencode compatibility */
 
   /*
    * In DOS, the default is to create files, not to send it to stdout
@@ -843,16 +869,70 @@ main (int argc, char *argv[])
    * Check for environment variable called INEWS and override
    * compile-time option if present
    */
+
   if (getenv ("INEWS") && *(char *)getenv ("INEWS")) {
     uue_inewsprog = getenv ("INEWS");
   }
 
+  /*
+   * Check for environment variable called UUENVIEW and read it
+   */
+
+  myargc = 1;
+  myargv[0] = argv[0];
+
+  if ((p1 = (char *) getenv ("UUENVIEW"))) {
+    myargv[myargc++] = p1;
+    quote = 0;
+
+    while (*p1 && myargc < 255) {
+      switch (*p1) {
+      case ' ':
+      case '\t':
+	if (!quote) {
+	  if ((*myargv[myargc-1] == '"' || *myargv[myargc-1] == '\'') &&
+	      p1-1 != myargv[myargc-1] &&
+	      *(p1-1) == *myargv[myargc-1]) {
+	    *(p1-1) = '\0';
+	    myargv[myargc-1] += 1;
+	  }
+	  *p1++ = '\0';
+	  while (*p1 == ' ' || *p1 == '\t')
+	    p1++;
+	  myargv[myargc++] = p1;
+	}
+	else {
+	  p1++;
+	}
+	break;
+
+      case '"':
+      case '\'':
+	if (!quote) {
+	  quote = *p1++;
+	}
+	else if (quote == *p1++) {
+	  quote = 0;
+	}
+	break;
+	
+      default:
+	p1++;
+      }
+    }
+  }
+
+  for (index=1; index<argc; index++) {
+    myargv[myargc++] = argv[index];
+  }
+
   /*
    * browse command line flags
    */
-  for (index=1; index<argc; index++) {
-    if (*argv[index] == '-') {
-      switch (argv[index][1]) {
+
+  for (index=1; index<myargc; index++) {
+    if (*myargv[index] == '-') {
+      switch (myargv[index][1]) {
       case '\0': /* read from stdin */
 	if (attach) {
 	  fprintf (stderr, "error: stdin already in use for attachment\n");
@@ -863,13 +943,17 @@ main (int argc, char *argv[])
 	  stdinused--;
 	}
 	stdinused++;
+	files++;
 	break;
       case 'b': encoding = B64ENCODED; break;
       case 'u': encoding = UU_ENCODED; break;
       case 'x': encoding = XX_ENCODED; break;
-#ifndef SYSTEM_DOS
+      case 't': encoding = PT_ENCODED; break;
+      case 'q': encoding = QP_ENCODED; break;
+      case 'y': encoding = YENC_ENCODED; break;
+#ifdef HAVE_POPEN
       case 'm': 
-	if (index+1<argc && argv[index+1][0] != '-') {
+	if (index+1<myargc && myargv[index+1][0] != '-') {
 	  outflags[UUE_MAILTO] = ++index; 
 	  tostdout             = 0;
 	}
@@ -877,7 +961,7 @@ main (int argc, char *argv[])
 	  fprintf (stderr, "error: -m requires a parameter\n");
 	break;
       case 'p':
-	if (index+1<argc && argv[index+1][0] != '-') {
+	if (index+1<myargc && myargv[index+1][0] != '-') {
 	  outflags[UUE_POSTTO] = ++index;
 	  tostdout             = 0;
 	}
@@ -886,9 +970,9 @@ main (int argc, char *argv[])
 	break;
 #endif
       case 'o':
-	if (argv[index][2] == 'd') {
-	  if (index+1<argc && argv[index+1][0] != '-') {
-	    strcpy (outdir, argv[++index]);
+	if (myargv[index][2] == 'd') {
+	  if (index+1<myargc && myargv[index+1][0] != '-') {
+	    strcpy (outdir, myargv[++index]);
 	    outflags[UUE_TOFILE] = index;
 	    tostdout             = 0;
 	    if (strlen(outdir)>0) {
@@ -907,20 +991,27 @@ main (int argc, char *argv[])
 	  tostdout             = 0;
 	}
 	break;
-      case 'i':
-	if (index+1<argc && argv[index+1][0] != '-') {
-	  introfile = ++index;
-	}
-	else
-	  fprintf (stderr, "error: -i requires a parameter\n");
-	break;
       case 's':
-	if (index+1<argc && argv[index+1][0] != '-') {
+	if (index+1<myargc && myargv[index+1][0] != '-') {
 	  subject   = ++index;
 	}
 	else
 	  fprintf (stderr, "error: -s requires a parameter\n");
 	break;
+      case 'f':
+	if (index+1<myargc && myargv[index+1][0] != '-') {
+	  from = ++index;
+	}
+	else
+	  fprintf (stderr, "error: -f requires a parameter\n");
+	break;
+      case 'r':
+	if (index+1<myargc && myargv[index+1][0] != '-') {
+	  replyto = ++index;
+	}
+	else
+	  fprintf (stderr, "error: -r requires a parameter\n");
+	break;
       case 'v':
 	UUSetOption (UUOPT_VERBOSE, 1, NULL);
 	break;
@@ -933,7 +1024,7 @@ main (int argc, char *argv[])
 	break;
       case 'h':
       case '?':
-	usage(argv[0]);
+	usage(myargv[0]);
 	return 0;
       case 'V':
 	fprintf (stdout, "uuenview %spl%s compiled on %s\n",
@@ -943,37 +1034,57 @@ main (int argc, char *argv[])
 	/* should be a line count of the form -1000 or a forced line count
 	 * like --100 which disables the sanity check
 	 */
-	if (argv[index][1]>='0' && argv[index][1]<='9') {
+	if (myargv[index][1]>='0' && myargv[index][1]<='9') {
 	  if (attach) {
 	    fprintf (stderr, "error: cannot attach to splitfile\n");
 	    return 1;
 	  }
-	  linperfile = atoi (argv[index] + 1);
+	  linperfile = atoi (myargv[index] + 1);
 	  if (linperfile != 0 && linperfile < 200) {
 	    fprintf (stderr,
 		     "warning: lines per file must be >= 200 (ignored).\n");
 	    linperfile=0;
 	  }
 	}
-	else if (argv[index][1]=='-' &&
-		 argv[index][2]>='0' && argv[index][2]<='9') {
+	else if (myargv[index][1]=='-' &&
+		 myargv[index][2]>='0' && myargv[index][2]<='9') {
 	  if (attach) {
 	    fprintf (stderr, "error: cannot attach to splitfile\n");
 	    return 1;
 	  }
-	  linperfile = atoi (argv[index] + 2);
+	  linperfile = atoi (myargv[index] + 2);
 	}
-	else if (argv[index][1]=='-') {
-	  usage(argv[0]);
+	else if (myargv[index][1]=='-') {
+	  usage(myargv[0]);
 	  return 0;
 	}
 	else {
 	  fprintf (stderr, "warning: unknown option '%s' ignored.\n",
-		   argv[index]);
+		   myargv[index]);
 	}
       }
     }
+    else {
+      files++;
+    }
   }
+
+  if (_FP_stristr (myargv[0], "uuencode") != NULL) {
+    uuencode = 1; /* uuencode compatibility */
+
+    if (encoding == -1) {
+      encoding = UU_ENCODED;
+    }
+
+    if (files == 1) {
+      stdinused = 1;
+    }
+    else if (files > 2) {
+      fprintf (stderr, "usage: %s [infile] destname\n", myargv[0]);
+      return 0;
+    }
+  }
+
   /*
    * check integrity
    */
@@ -982,25 +1093,24 @@ main (int argc, char *argv[])
     fprintf (stderr, "warning: cannot split file on standard output (use -o)\n");
     linperfile=0;
   }
-  if (introfile>0 && tostdout) {
-    fprintf (stderr, "warning: -i not possible on standard output (use -m or -p)\n");
-    introfile = -1;
-  }
-  if (introfile>0) {
-    if ((testit = fopen (argv[introfile], "r")) == NULL) {
-      fprintf (stderr, "warning: introfile %s not readable.\n",
-	       argv[introfile]);
-      introfile = -1;
-    }
-    else
-      fclose (testit);
-  }
+
   if (subject>0 && tostdout && !attach) {
     fprintf (stderr, "warning: -s not possible on standard output\n");
     subject = -1;
   }
+
+  if (encoding == -1) {
+    if (attach || outflags[UUE_POSTTO]>0 || outflags[UUE_MAILTO]>0) {
+      encoding = B64ENCODED;
+    }
+    else {
+      encoding = UU_ENCODED;
+    }
+  }
+
   if (encoding!=UU_ENCODED && encoding!=XX_ENCODED &&
-      encoding!=B64ENCODED) {
+      encoding!=B64ENCODED && encoding!=PT_ENCODED &&
+      encoding!=QP_ENCODED && encoding!=YENC_ENCODED) {
     fprintf (stderr, "warning: unknown encoding method (%d)?\n",
 	     encoding);
     encoding=UU_ENCODED;
@@ -1027,18 +1137,25 @@ main (int argc, char *argv[])
       outflags[UUE_POSTTO] = 0;
     }
     if (outflags[UUE_POSTTO]>0) {
-      AttachFiles (argv[outflags[UUE_POSTTO]],
-		   (subject>0)?argv[subject]:NULL,
-		   0, encoding, argc, argv);
+      AttachFiles (myargv[outflags[UUE_POSTTO]],
+		   (subject>0)?myargv[subject]:NULL,
+		   (from>0)?myargv[from]:NULL,
+		   (replyto>0)?myargv[replyto]:NULL,
+		   0, encoding, myargc, myargv);
     }
     else if (outflags[UUE_MAILTO]>0) {
-      AttachFiles (argv[outflags[UUE_MAILTO]],
-		   (subject>0)?argv[subject]:NULL,
-		   1, encoding, argc, argv);
+      AttachFiles (myargv[outflags[UUE_MAILTO]],
+		   (subject>0)?myargv[subject]:NULL,
+		   (from>0)?myargv[from]:NULL,
+		   (replyto>0)?myargv[replyto]:NULL,
+		   1, encoding, myargc, myargv);
     }
     else /* tostdout */ {
-      AttachFiles (NULL, (subject>0)?argv[subject]:NULL,
-		   0, encoding, argc, argv);
+      AttachFiles (NULL,
+		   (subject>0)?myargv[subject]:NULL,
+		   (from>0)?myargv[from]:NULL,
+		   (replyto>0)?myargv[replyto]:NULL,
+		   0, encoding, myargc, myargv);
     }
     /* finished here */
     count=1;
@@ -1049,22 +1166,30 @@ main (int argc, char *argv[])
    * okay, now process the files
    */
 
-  for (index=1, iskipflag=0; index<argc; index+=iskipflag+1) {
+  for (index=1, iskipflag=0; index<myargc; index+=iskipflag+1) {
     iskipflag = 0;
 
-    if (*argv[index] == '-' && argv[index][1] != '\0') {
-      switch (argv[index][1]) {
+    if (*myargv[index] == '-' && myargv[index][1] != '\0') {
+      switch (myargv[index][1]) {
       case 'm': /* skip parameters of options */
       case 'p':
       case 's':
       case 'i':
-	if (index+1 < argc && argv[index+1][0] != '-')
+      case 'f':
+      case 'r':
+	if (index+1 < myargc && myargv[index+1][0] != '-')
 	  index++;
 	break;
       case 'o': /* may or may have not a parameter */
-	if (argv[index][2]=='d' && index+1<argc && argv[index+1][0]!='-')
+	if (myargv[index][2]=='d' && index+1<myargc && myargv[index+1][0]!='-')
 	  index++;
 	break;
+      case 'b': encoding = B64ENCODED; break;
+      case 'u': encoding = UU_ENCODED; break;
+      case 'x': encoding = XX_ENCODED; break;
+      case 't': encoding = PT_ENCODED; break;
+      case 'q': encoding = QP_ENCODED; break;
+      case 'y': encoding = YENC_ENCODED; break;
       default:  /* parameter without option */
 	break;
       }
@@ -1078,9 +1203,15 @@ main (int argc, char *argv[])
 	 * file. If there is no next parameter, we shall read from stdin
 	 * and use the given name as output filename
 	 */
-	if (index+1<argc && argv[index+1][0] != '-') {
-	  strcpy (filename, argv[index]);
-	  strcpy (usename,  argv[index+1]);
+	if (index+1<myargc && myargv[index+1][0] != '-') {
+	  if (myargv[index][0] != '-' || myargv[index][1] != '\0') {
+	    strcpy (filename, myargv[index]);
+	  }
+	  else {
+	    filename[0] = '\0';
+	    stdinused--;
+	  }
+	  strcpy (usename, myargv[index+1]);
 	  iskipflag = 1; /* ignore next argument */
 	}
 	else {
@@ -1090,10 +1221,10 @@ main (int argc, char *argv[])
 	  }
 	  stdinused--;
 	  filename[0] = '\0';
-	  strcpy (usename, argv[index]);
+	  strcpy (usename, myargv[index]);
 	}
       }
-      else if (argv[index][0]=='-' && argv[index][1]=='\0') {
+      else if (myargv[index][0]=='-' && myargv[index][1]=='\0') {
 	/*
 	 * supposed to read from stdin. we expect the next parameter to be
 	 * the name to be used
@@ -1104,8 +1235,8 @@ main (int argc, char *argv[])
 	}
 	stdinused--;
 
-	if (index+1<argc && argv[index+1][0] != '-') {
-	  strcpy (usename, argv[index+1]);
+	if (index+1<myargc && myargv[index+1][0] != '-') {
+	  strcpy (usename, myargv[index+1]);
 	  filename[0] = '\0';
 	  iskipflag = 1; /* ignore next argument */
 	}
@@ -1115,7 +1246,7 @@ main (int argc, char *argv[])
 	}
       }
       else {
-	strcpy (filename, argv[index]);
+	strcpy (filename, myargv[index]);
 	strcpy (usename,  filename);
       }
 
@@ -1127,8 +1258,8 @@ main (int argc, char *argv[])
 	  continue;
 	}
       }
-      else if ((testit = fopen (argv[index], "r")) == NULL) {
-	fprintf (stderr, "error: '%s' unreadable.\n", argv[index]);
+      else if ((testit = fopen (myargv[index], "r")) == NULL) {
+	fprintf (stderr, "error: '%s' unreadable.\n", myargv[index]);
 	continue;
       }
       else
@@ -1136,18 +1267,21 @@ main (int argc, char *argv[])
 
       fileflag = 0;
 
-#ifndef SYSTEM_DOS
+#ifdef HAVE_POPEN
       /*
        * post it
        */
 
       if (outflags[UUE_POSTTO] > 0) {
-	if (SendAFile ((filename[0])?NULL:stdin,
-		       (filename[0])?filename:NULL,
-		       encoding, linperfile, usename,
-		       argv[outflags[UUE_POSTTO]],
-		       (subject>0)?argv[subject]:NULL, 0) == UURET_OK)
-	  fileflag++;
+	fileflag++;
+	SendAFile ((filename[0])?NULL:stdin,
+		   (filename[0])?filename:NULL,
+		   encoding, linperfile, usename,
+		   myargv[outflags[UUE_POSTTO]],
+		   (subject>0)?myargv[subject]:NULL,
+		   (from>0)?myargv[from]:NULL,
+		   (replyto>0)?myargv[replyto]:NULL,
+		   0);
       }
 
       /*
@@ -1155,12 +1289,15 @@ main (int argc, char *argv[])
        */
 
       if (outflags[UUE_MAILTO] > 0) {
-	if (SendAFile ((filename[0])?NULL:stdin,
-		       (filename[0])?filename:NULL,
-		       encoding, linperfile, usename,
-		       argv[outflags[UUE_MAILTO]],
-		       (subject>0)?argv[subject]:NULL, 1) == UURET_OK)
-	  fileflag++;
+	fileflag++;
+	SendAFile ((filename[0])?NULL:stdin,
+		   (filename[0])?filename:NULL,
+		   encoding, linperfile, usename,
+		   myargv[outflags[UUE_MAILTO]],
+		   (subject>0)?myargv[subject]:NULL,
+		   (from>0)?myargv[from]:NULL,
+		   (replyto>0)?myargv[replyto]:NULL,
+		   1);
       }
 #endif
 
@@ -1168,22 +1305,22 @@ main (int argc, char *argv[])
        * store output into a file
        */
       if (outflags[UUE_TOFILE] > 0 || outflags[UUE_TOFILE] == -42) {
-	if (UUEncodeToFile ((filename[0])?NULL:stdin,
-			    (filename[0])?filename:NULL,
-			    encoding, usename,
-			    NULL, linperfile) == UURET_OK)
-	  fileflag++;
+	fileflag++;
+	UUEncodeToFile ((filename[0])?NULL:stdin,
+			(filename[0])?filename:NULL,
+			encoding, usename,
+			NULL, linperfile);
       }
 
       /*
        * send it to stdout
        */
       if (tostdout) {
-	if (UUEncodeToStream (stdout,
-			      (filename[0])?NULL:stdin,
-			      (filename[0])?filename:NULL,
-			      encoding, usename, 0) == UURET_OK)
-	  fileflag++;
+	fileflag++;
+	UUEncodeToStream (stdout,
+			  (filename[0])?NULL:stdin,
+			  (filename[0])?filename:NULL,
+			  encoding, usename, 0);
       }
 
       if (fileflag > 0)
