@@ -44,7 +44,7 @@
 #include <uufnflt.h>
 
 
-char * uufnflt_id = "$Id: uufnflt.c,v 1.4 2001/06/06 18:21:40 fp Exp $";
+char * uufnflt_id = "$Id: uufnflt.c,v 1.7 2003/04/13 15:41:54 fp Exp $";
 
 char *
 UUFNameFilterUnix (void *opaque, char *fname)
@@ -58,11 +58,47 @@ UUFNameFilterUnix (void *opaque, char *fname)
    * strip directory information
    */
 
-  if ((ptr = _FP_strrchr (fname, '/')) != NULL)
-    return ptr+1;
-  else if ((ptr = _FP_strrchr (fname, '\\')) != NULL)
-    return ptr+1;
+  if ((ptr = _FP_strrchr (fname, '/')) != NULL) {
+    fname = ptr+1;
+  }
 
+  if ((ptr = _FP_strrchr (fname, '\\')) != NULL) {
+    fname = ptr+1;
+  }
+
+  if (strcmp (fname, ".") == 0 || strcmp (fname, "..") == 0) {
+    fname[0] = '_';
+  }
+
+  /*
+   * remove space characters from the beginning
+   */
+
+  while (*fname == ' ') {
+    fname++;
+  }
+
+  /*
+   * If filename is empty by now, return a dummy string
+   */
+
+  if (!*fname) {
+    return "blank_filename";
+  }
+
+  /*
+   * Replace all questionable punctuation characters by underscores
+   */
+
+  for (ptr=fname; *ptr; ptr++) {
+    if (*ptr == '"' || *ptr == '<' || *ptr == '>' ||
+	*ptr == '{' || *ptr == '}' || *ptr == '|' ||
+	*ptr == '&' || *ptr == '\t' || *ptr == '\r' ||
+	*ptr == '\n') {
+      *ptr = '_';
+    }
+  }
+  
   return fname;
 }
 
