@@ -53,7 +53,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uucheck_id = "$Id: uucheck.c,v 1.13 2002/02/28 08:30:02 fp Exp $";
+char * uucheck_id = "$Id: uucheck.c,v 1.15 2003/04/13 15:41:55 fp Exp $";
 
 /*
  * Arbitrary number. This is the maximum number of part numbers we
@@ -775,7 +775,7 @@ UUPreProcessPart (fileread *data, int *ret)
     /*
      * in this case, it really _should_ have a filename somewhere
      */
-    if (result->filename != NULL)
+    if (result->filename != NULL && *result->filename)
       result->subfname = _FP_strdup (result->filename);
     else { /* if not, escape to UNKNOWN. We need to fill subfname */
       sprintf (temp, "%s.%03d", nofname, ++nofnum);
@@ -829,7 +829,7 @@ UUPreProcessPart (fileread *data, int *ret)
       /*
        * Assume it's the first part. I wonder why it's got no part number?
        */
-      if (result->filename != NULL)
+      if (result->filename != NULL && *result->filename)
         result->subfname = _FP_strdup (result->filename);
       else { /* if not, escape to UNKNOWN. We need to fill subfname */
         sprintf (temp, "%s.%03d", nofname, ++nofnum);
@@ -976,13 +976,13 @@ UUInsertPartToList (uufile *data)
 
   /*
    * Part belongs together, if
-   * (a) The file name received from the subject lines match _or_
-   *     the MIME-IDs match,
-   * (b) Not both parts have a begin line
-   * (c) Not both parts have an end line
-   * (d) Both parts don't have different MIME-IDs
-   * (e) Both parts don't encode different files
-   * (f) The other part wants to stay alone (FL_SINGLE)
+   * (1) The MIME-IDs match, or
+   * (2) The file name received from the subject lines match, and
+   *     (a) Not both parts have a begin line
+   *     (b) Not both parts have an end line
+   *     (c) Both parts don't have different MIME-IDs
+   *     (d) Both parts don't encode different files
+   *     (e) The other part wants to stay alone (FL_SINGLE)
    */
 
   /*
@@ -994,29 +994,33 @@ UUInsertPartToList (uufile *data)
     if (data->data->flags & FL_SINGLE) {
       /* this space intentionally left blank */
     }
-    else if ((_FP_stricmp (data->subfname, iter->subfname) == 0 ||
-         (data->mimeid && iter->mimeid &&
-          strcmp (data->mimeid, iter->mimeid) == 0)) &&
-	!(iter->begin && data->data->begin) &&
-	!(iter->end   && data->data->end) &&
-	!(data->mimeid && iter->mimeid &&
-	  strcmp (data->mimeid, iter->mimeid) != 0) &&
-	!(data->filename && iter->filename &&
-	  strcmp (data->filename, iter->filename) != 0) &&
-	!(iter->flags & FL_SINGLE)) {
+    else if ((data->mimeid && iter->mimeid &&
+	      strcmp (data->mimeid, iter->mimeid) == 0) ||
+	     (_FP_stricmp (data->subfname, iter->subfname) == 0 &&
+	      !(iter->begin && data->data->begin) &&
+	      !(iter->end   && data->data->end) &&
+	      !(data->mimeid && iter->mimeid &&
+		strcmp (data->mimeid, iter->mimeid) != 0) &&
+	      !(data->filename && iter->filename &&
+		strcmp (data->filename, iter->filename) != 0) &&
+	      !(iter->flags & FL_SINGLE))) {
 
       /*
-       * if we already have this part, don't try to insert it
+       * Don't insert a part that is already there.
+       *
+       * Also don't add a part beyond the "end" marker (unless we
+       * have a mimeid, which screws up the marker).
        */
 
-      for (fiter=iter->thisfile;
-	   fiter && (data->partno>fiter->partno) && !fiter->data->end;
-	   fiter=fiter->NEXT)
-	/* empty loop */ ;
-      if (fiter && 
-	  (data->partno==fiter->partno || 
-	   (data->partno > fiter->partno && fiter->data->end)))
-	goto goahead;
+      for (fiter=iter->thisfile; fiter; fiter=fiter->NEXT) {
+	if (data->partno == fiter->partno)
+	  goto goahead;
+	if (!iter->mimeid) {
+	  if (data->partno > fiter->partno && fiter->data->end) {
+	    goto goahead;
+	  }
+	}
+      }
 
       if (iter->filename == NULL && data->filename != NULL) {
         if ((iter->filename = _FP_strdup (data->filename)) == NULL)
