@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
+#include <crc32.h>
 
 /* for braindead systems */
 #ifndef SEEK_SET
@@ -56,7 +57,7 @@
 #endif
 #endif
 
-char * uuencode_id = "$Id: uuencode.c,v 1.15 1996/09/30 19:28:16 fp Exp $";
+char * uuencode_id = "$Id: uuencode.c,v 1.22 2002/04/02 10:04:52 fp Exp $";
 
 #if 0
 /*
@@ -110,11 +111,15 @@ static unsigned char *eolstring = (unsigned char *) "\012";
 #define CTE_UUENC	"x-uuencode"
 #define CTE_XXENC	"x-xxencode"
 #define CTE_BINHEX	"x-binhex"
+#define CTE_YENC	"x-yenc"
 
 #define CTE_TYPE(y)	(((y)==B64ENCODED) ? "Base64"  : \
 			 ((y)==UU_ENCODED) ? CTE_UUENC : \
 			 ((y)==XX_ENCODED) ? CTE_XXENC : \
-			 ((y)==BH_ENCODED) ? CTE_BINHEX : "x-oops")
+                         ((y)==PT_ENCODED) ? "8bit" : \
+                         ((y)==QP_ENCODED) ? "quoted-printable" : \
+			 ((y)==BH_ENCODED) ? CTE_BINHEX : \
+			 ((y)==YENC_ENCODED) ? CTE_YENC : "x-oops")
 
 /*
  * encoding tables
@@ -163,7 +168,12 @@ unsigned char BHEncodeTable[64] = {
   'S', 'T', 'U', 'V', 'X', 'Y', 'Z', '[', 
   '`', 'a', 'b', 'c', 'd', 'e', 'f', 'h', 
   'i', 'j', 'k', 'l', 'm', 'p', 'q', 'r'
-};    
+};
+
+unsigned char HexEncodeTable[16] = {
+  '0', '1', '2', '3', '4', '5', '6', '7',
+  '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
+};
 
 typedef struct {
   char *extension;
@@ -191,7 +201,7 @@ static mimemap mimetable[] = {
 	{ "mpeg", "video/mpeg"       }, /* Motion Picture Expert Group */
 	{ "mpg",  "video/mpeg"       },
 	{ "mp2",  "video/mpeg"       }, /* dito, MPEG-2 encoded files  */
-	{ "mp3",  "video/mpeg"       }, /* dito, MPEG-3 encoded files  */
+	{ "mp3",  "audio/mpeg"       }, /* dito, MPEG-3 encoded files  */
 	{ "ps",   "application/postscript" }, /* Postscript Language   */
 	{ "zip",  "application/zip"  }, /* ZIP archive                 */
 	{ "doc",  "application/msword"},/* assume Microsoft Word       */
@@ -207,7 +217,7 @@ static mimemap mimetable[] = {
  * encoded bytes per line
  */
 
-static int bpl[5] = { 0, 45, 57, 45, 45 };
+static int bpl[8] = { 0, 45, 57, 45, 45, 0, 0, 128 };
 
 /*
  * tables
@@ -233,7 +243,7 @@ char *uuestr_otemp;
  */
 
 static int 
-UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
+UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile, crc32_t *crc, crc32_t *pcrc)
 {
   unsigned char *itemp = (char *) uuestr_itemp;
   unsigned char *otemp = (char *) uuestr_otemp;
@@ -243,12 +253,254 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
   size_t llen;
 
   if (outfile==NULL || infile==NULL ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUEncodeStream()");
     return UURET_ILLVAL;
   }
 
+  /*
+   * Special handling for plain text and quoted printable. Text is
+   * read line oriented.
+   */
+
+  if (encoding == PT_ENCODED || encoding == QP_ENCODED) {
+    while (!feof (infile) && (linperfile <= 0 || line < linperfile)) {
+      if (_FP_fgets (itemp, 255, infile) == NULL) {
+	break;
+      }
+
+      itemp[255] = '\0';
+      count = strlen (itemp);
+
+      llen = 0;
+      optr = otemp;
+
+      /*
+       * Busy Callback
+       */
+      
+      if (UUBUSYPOLL(ftell(infile)-progress.foffset,progress.fsize)) {
+	UUMessage (uuencode_id, __LINE__, UUMSG_NOTE,
+		   uustring (S_ENCODE_CANCEL));
+	return UURET_CANCEL;
+      }
+
+      if (encoding == PT_ENCODED) {
+	/*
+	 * If there is a line feed, replace by eolstring
+	 */
+	if (count > 0 && itemp[count-1] == '\n') {
+	  itemp[--count] = '\0';
+	  if (fwrite (itemp, 1, count, outfile) != count ||
+	      fwrite ((char *) eolstring, 1,
+		      strlen(eolstring), outfile) != strlen (eolstring)) {
+	    return UURET_IOERR;
+	  }
+	}
+	else {
+	  if (fwrite (itemp, 1, count, outfile) != llen) {
+	    return UURET_IOERR;
+	  }
+	}
+      }
+      else if (encoding == QP_ENCODED) {
+	for (index=0; index<count; index++) {
+	  if (llen == 0 && itemp[index] == '.') {
+	    /*
+	     * Special rule: encode '.' at the beginning of a line, so
+	     * that some mailers aren't confused.
+	     */
+	    *optr++ = '=';
+	    *optr++ = HexEncodeTable[itemp[index] >> 4];
+	    *optr++ = HexEncodeTable[itemp[index] & 0x0f];
+	    llen += 3;
+	  }
+	  else if ((itemp[index] >= 33 && itemp[index] <= 60) ||
+		   (itemp[index] >= 62 && itemp[index] <= 126) ||
+		   itemp[index] == 9 || itemp[index] == 32) {
+	    *optr++ = itemp[index];
+	    llen++;
+	  }
+	  else if (itemp[index] == '\n') {
+	    /*
+	     * If the last character before EOL was a space or tab,
+	     * we must encode it. If llen > 74, there's no space to do
+	     * that, so generate a soft line break instead.
+	     */
+
+	    if (index>0 && (itemp[index-1] == 9 || itemp[index-1] == 32)) {
+	      *(optr-1) = '=';
+	      if (llen <= 74) {
+		*optr++ = HexEncodeTable[itemp[index-1] >> 4];
+		*optr++ = HexEncodeTable[itemp[index-1] & 0x0f];
+		llen += 2;
+	      }
+	    }
+
+	    if (fwrite (otemp, 1, llen, outfile) != llen ||
+		fwrite ((char *) eolstring, 1,
+			strlen(eolstring), outfile) != strlen (eolstring)) {
+	      return UURET_IOERR;
+	    }
+
+	    /*
+	     * Fix the soft line break condition from above
+	     */
+
+	    if (index>0 && (itemp[index-1] == 9 || itemp[index-1] == 32) &&
+		*(optr-1) == '=') {
+	      otemp[0] = '=';
+	      otemp[1] = HexEncodeTable[itemp[index-1] >> 4];
+	      otemp[2] = HexEncodeTable[itemp[index-1] & 0x0f];
+
+	      if (fwrite (otemp, 1, 3, outfile) != 3 ||
+		  fwrite ((char *) eolstring, 1,
+			  strlen(eolstring), outfile) != strlen (eolstring)) {
+		return UURET_IOERR;
+	      }
+	    }
+
+	    optr = otemp;
+	    llen = 0;
+	  }
+	  else {
+	    *optr++ = '=';
+	    *optr++ = HexEncodeTable[itemp[index] >> 4];
+	    *optr++ = HexEncodeTable[itemp[index] & 0x0f];
+	    llen += 3;
+	  }
+
+	  /*
+	   * Lines must be shorter than 76 characters (not counting CRLF).
+	   * If the line grows longer than that, we must include a soft
+	   * line break.
+	   */
+
+	  if (itemp[index+1] != 0 && itemp[index+1] != '\n' &&
+	      (llen >= 75 ||
+	       (!((itemp[index+1] >= 33 && itemp[index+1] <= 60) ||
+		  (itemp[index+1] >= 62 && itemp[index+1] <= 126)) &&
+		llen >= 73))) {
+
+	    *optr++ = '=';
+	    llen++;
+	    
+	    if (fwrite (otemp, 1, llen, outfile) != llen ||
+		fwrite ((char *) eolstring, 1,
+			strlen(eolstring), outfile) != strlen (eolstring)) {
+	      return UURET_IOERR;
+	    }
+	    
+	    optr = otemp;
+	    llen = 0;
+	  }
+	}
+      }
+
+      line++;
+    }
+
+    return UURET_OK;
+  }
+
+  /*
+   * Special handling for yEnc
+   */
+
+  if (encoding == YENC_ENCODED) {
+    llen = 0;
+    optr = otemp;
+
+    while (!feof (infile) && (linperfile <= 0 || line < linperfile)) {
+      if ((count = fread (itemp, 1, 128, infile)) != 128) {
+	if (count == 0) {
+	  break;
+	}
+	else if (ferror (infile)) {
+	  return UURET_IOERR;
+	}
+      }
+
+      if (pcrc)
+	*pcrc = crc32(*pcrc, itemp, count);
+      if (crc)
+	*crc = crc32(*crc, itemp, count);
+
+      line++;
+
+      /*
+       * Busy Callback
+       */
+      
+      if (UUBUSYPOLL(ftell(infile)-progress.foffset,progress.fsize)) {
+	UUMessage (uuencode_id, __LINE__, UUMSG_NOTE,
+		   uustring (S_ENCODE_CANCEL));
+	return UURET_CANCEL;
+      }
+
+      for (index=0; index<count; index++) {
+	if (llen > 127) {
+	  if (fwrite (otemp, 1, llen, outfile) != llen ||
+	      fwrite ((char *) eolstring, 1,
+		      strlen(eolstring), outfile) != strlen (eolstring)) {
+	    return UURET_IOERR;
+	  }
+	  llen = 0;
+	  optr = otemp;
+	}
+
+	switch ((char) ((int) itemp[index] + 42)) {
+	case '\0':
+	case '\t':
+	case '\n':
+	case '\r':
+	case '=':
+	case '\033':
+	  *optr++ = '=';
+	  *optr++ = (char) ((int) itemp[index] + 42 + 64);
+	  llen += 2;
+	  break;
+
+	case '.':
+	  if (llen == 0) {
+	    *optr++ = '=';
+	    *optr++ = (char) ((int) itemp[index] + 42 + 64);
+	    llen += 2;
+	  }
+	  else {
+	    *optr++ = (char) ((int) itemp[index] + 42);
+	    llen++;
+	  }
+	  break;
+
+	default:
+	  *optr++ = (char) ((int) itemp[index] + 42);
+	  llen++;
+	  break;
+	}
+      }
+    }
+
+    /*
+     * write last line
+     */
+
+    if (llen) {
+      if (fwrite (otemp, 1, llen, outfile) != llen ||
+	  fwrite ((char *) eolstring, 1,
+		  strlen(eolstring), outfile) != strlen (eolstring)) {
+	return UURET_IOERR;
+      }
+    }
+
+    return UURET_OK;
+  }
+
+  /*
+   * Handling for binary encodings
+   */
 
   /*
    * select charset
@@ -258,7 +510,7 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
 
   if (table==NULL || bpl[encoding]==0) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
-	       uustring (S_PARM_CHECK), "UUEncodeStream()");
+               uustring (S_PARM_CHECK), "UUEncodeStream()");
     return UURET_ILLVAL;
   }
 
@@ -269,6 +521,7 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
       else if (ferror (infile))
 	return UURET_IOERR;
     }
+
     optr = otemp;
     llen = 0;
 
@@ -285,21 +538,27 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
     /*
      * for UU and XX, encode the number of bytes as first character
      */
+
     if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
       *optr++ = table[count];
       llen++;
     }
 
+    /*
+     * Main encoding
+     */
+
     for (index=0; index<=count-3; index+=3, llen+=4) {
       *optr++ = table[itemp[index] >> 2];
-      *optr++ = table[((itemp[index  ] & 0x03) << 4) | (itemp[index+1] >> 4)];
-      *optr++ = table[((itemp[index+1] & 0x0f) << 2) | (itemp[index+2] >> 6)];
+      *optr++ = table[((itemp[index  ] & 0x03) << 4)|(itemp[index+1] >> 4)];
+      *optr++ = table[((itemp[index+1] & 0x0f) << 2)|(itemp[index+2] >> 6)];
       *optr++ = table[  itemp[index+2] & 0x3f];
     }
 
     /*
      * Special handling for incomplete lines
      */
+
     if (index != count) {
       if (encoding == B64ENCODED) {
 	if (count - index == 2) {
@@ -317,7 +576,7 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
 	}
 	llen += 4;
       }
-      else {
+      else if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
 	if (count - index == 2) {
 	  *optr++ = table[itemp[index] >> 2];
 	  *optr++ = table[((itemp[index  ] & 0x03) << 4) | 
@@ -334,9 +593,11 @@ UUEncodeStream (FILE *outfile, FILE *infile, int encoding, long linperfile)
 	llen += 4;
       }
     }
+
     /*
      * end of line
      */
+
     tptr = eolstring;
 
     while (*tptr)
@@ -366,11 +627,14 @@ UUEncodeMulti (FILE *outfile, FILE *infile, char *infname, int encoding,
   int res, themode;
   FILE *theifile;
   char *ptr;
+  crc32_t crc;
+  crc32_t *crcptr=NULL;
 
   if (outfile==NULL || 
       (infile == NULL && infname==NULL) ||
       (outfname==NULL && infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUEncodeMulti()");
     return UURET_ILLVAL;
@@ -406,7 +670,7 @@ UUEncodeMulti (FILE *outfile, FILE *infile, char *infname, int encoding,
     theifile = infile;
   }
 
-  if (progress.fsize <= 0)
+  if (progress.fsize < 0)
     progress.fsize = -1;
 
   _FP_strncpy (progress.curfile, (outfname)?outfname:infname, 256);
@@ -430,18 +694,25 @@ UUEncodeMulti (FILE *outfile, FILE *infile, char *infname, int encoding,
       mimetype = miter->mimetype;
     }
   }
+
+  if (mimetype == NULL && (encoding == PT_ENCODED || encoding == QP_ENCODED)) {
+    mimetype = "text/plain";
+  }
+
   /*
    * print sub-header
    */
 
-  fprintf (outfile, "Content-Type: %s%s",
-	   (mimetype)?mimetype:"Application/Octet-Stream",
-	   eolstring);
-  fprintf (outfile, "Content-Transfer-Encoding: %s%s",
-	   CTE_TYPE(encoding), eolstring);
-  fprintf (outfile, "Content-Disposition: attachment; filename=\"%s\"%s",
-	   UUFNameFilter ((outfname)?outfname:infname), eolstring);
-  fprintf (outfile, "%s", eolstring);
+  if (encoding != YENC_ENCODED) {
+    fprintf (outfile, "Content-Type: %s%s",
+	     (mimetype)?mimetype:"Application/Octet-Stream",
+	     eolstring);
+    fprintf (outfile, "Content-Transfer-Encoding: %s%s",
+	     CTE_TYPE(encoding), eolstring);
+    fprintf (outfile, "Content-Disposition: attachment; filename=\"%s\"%s",
+	     UUFNameFilter ((outfname)?outfname:infname), eolstring);
+    fprintf (outfile, "%s", eolstring);
+  }
 
   if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
     fprintf (outfile, "begin %o %s%s",
@@ -449,7 +720,23 @@ UUEncodeMulti (FILE *outfile, FILE *infile, char *infname, int encoding,
 	     UUFNameFilter ((outfname)?outfname:infname), 
 	     eolstring);
   }
-  if ((res = UUEncodeStream (outfile, theifile, encoding, 0)) != UURET_OK) {
+  else if (encoding == YENC_ENCODED) {
+    crc = crc32(0L, Z_NULL, 0);
+    crcptr = &crc;
+    if (progress.fsize == -1) {
+      fprintf (outfile, "=ybegin line=128 name=%s%s",
+	       UUFNameFilter ((outfname)?outfname:infname), 
+	       eolstring);
+    }
+    else {
+      fprintf (outfile, "=ybegin line=128 size=%ld name=%s%s",
+	       progress.fsize,
+	       UUFNameFilter ((outfname)?outfname:infname), 
+	       eolstring);
+    }
+  }
+
+  if ((res = UUEncodeStream (outfile, theifile, encoding, 0, crcptr, NULL)) != UURET_OK) {
     if (res != UURET_CANCEL) {
       UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_ERR_ENCODING),
@@ -459,15 +746,31 @@ UUEncodeMulti (FILE *outfile, FILE *infile, char *infname, int encoding,
     progress.action = 0;
     return res;
   }
+
   if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
     fprintf (outfile, "%c%s",    
 	     (encoding==UU_ENCODED) ? UUEncodeTable[0] : XXEncodeTable[0], 
 	     eolstring);
     fprintf (outfile, "end%s", eolstring);
   }
+  else if (encoding == YENC_ENCODED) {
+    if (progress.fsize == -1) {
+      fprintf (outfile, "=yend crc32=%08lx%s",
+	       crc,
+	       eolstring);
+    }
+    else {
+      fprintf (outfile, "=yend size=%ld crc32=%08lx%s",
+	       progress.fsize,
+	       crc,
+	       eolstring);
+    }
+  }
+
   /*
    * empty line at end does no harm
    */
+
   fprintf (outfile, "%s", eolstring);
 
   if (infile==NULL)
@@ -485,7 +788,8 @@ int UUEXPORT
 UUEncodePartial (FILE *outfile, FILE *infile,
 		 char *infname, int encoding,
 		 char *outfname, char *mimetype,
-		 int filemode, int partno, long linperfile)
+		 int filemode, int partno, long linperfile,
+		 crc32_t *crcptr)
 {
   mimemap *miter=mimetable;
   static FILE *theifile;
@@ -494,10 +798,13 @@ UUEncodePartial (FILE *outfile, FILE *infile,
   long thesize;
   char *ptr;
   int res;
+  crc32_t pcrc;
+  crc32_t *pcrcptr=NULL;
 
   if ((outfname==NULL&&infname==NULL) || partno<=0 ||
       (infile == NULL&&infname==NULL) || outfile==NULL ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUEncodePartial()");
     return UURET_ILLVAL;
@@ -538,7 +845,7 @@ UUEncodePartial (FILE *outfile, FILE *infile,
 		   uustring (S_STAT_ONE_PART));
 	numparts = 1;
 	themode  = (filemode)?filemode:0644;
-	thesize  = 0;
+	thesize  = -1;
       }
       else {
 	if (linperfile <= 0)
@@ -555,7 +862,7 @@ UUEncodePartial (FILE *outfile, FILE *infile,
 
     _FP_strncpy (progress.curfile, (outfname)?outfname:infname, 256);
 
-    progress.totsize  = (thesize>0) ? thesize : -1;
+    progress.totsize  = (thesize>=0) ? thesize : -1;
     progress.partno   = 1;
     progress.numparts = numparts;
     progress.percent  = 0;
@@ -574,29 +881,76 @@ UUEncodePartial (FILE *outfile, FILE *infile,
 	mimetype = miter->mimetype;
       }
     }
+
+    if (mimetype == NULL && (encoding==PT_ENCODED || encoding==QP_ENCODED)) {
+      mimetype = "text/plain";
+    }
+
     /*
      * print sub-header
      */
 
-    fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
-    fprintf (outfile, "Content-Type: %s%s",
-	     (mimetype)?mimetype:"Application/Octet-Stream",
-	     eolstring);
-    fprintf (outfile, "Content-Transfer-Encoding: %s%s",
-	     CTE_TYPE(encoding), eolstring);
-    fprintf (outfile, "Content-Disposition: attachment; filename=\"%s\"%s",
-	     UUFNameFilter ((outfname)?outfname:infname), eolstring);
-    fprintf (outfile, "%s", eolstring);
+    if (encoding != YENC_ENCODED) {
+      fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
+      fprintf (outfile, "Content-Type: %s%s",
+	       (mimetype)?mimetype:"Application/Octet-Stream",
+	       eolstring);
+      fprintf (outfile, "Content-Transfer-Encoding: %s%s",
+	       CTE_TYPE(encoding), eolstring);
+      fprintf (outfile, "Content-Disposition: attachment; filename=\"%s\"%s",
+	       UUFNameFilter ((outfname)?outfname:infname), eolstring);
+    }
 
+    fprintf (outfile, "%s", eolstring);
+    
     /*
      * for the first part of UU or XX messages, print a begin line
      */
+
     if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
       fprintf (outfile, "begin %o %s%s",
 	       (themode) ? themode : ((filemode)?filemode:0644),
 	       UUFNameFilter ((outfname)?outfname:infname), eolstring);
     }
   }
+  if (encoding == YENC_ENCODED) {
+    pcrc = crc32(0L, Z_NULL, 0);
+    pcrcptr = &pcrc;
+    if (numparts != 1) {
+      if (progress.totsize == -1) {
+	fprintf (outfile, "=ybegin part=%d line=128 name=%s%s",
+		 partno,
+		 UUFNameFilter ((outfname)?outfname:infname), 
+		 eolstring);
+      }
+      else {
+	fprintf (outfile, "=ybegin part=%d line=128 size=%ld name=%s%s",
+		 partno,
+		 progress.totsize,
+		 UUFNameFilter ((outfname)?outfname:infname), 
+		 eolstring);
+      }
+
+      fprintf (outfile, "=ypart begin=%d end=%d%s",
+	       (partno-1)*linperfile*128+1,
+	       (partno*linperfile*128) < progress.totsize ? 
+	       (partno*linperfile*128) : progress.totsize,
+	       eolstring);
+    }
+    else {
+      if (progress.totsize == -1) {
+	fprintf (outfile, "=ybegin line=128 name=%s%s",
+		 UUFNameFilter ((outfname)?outfname:infname), 
+		 eolstring);
+      }
+      else {
+	fprintf (outfile, "=ybegin line=128 size=%ld name=%s%s",
+		 progress.totsize,
+		 UUFNameFilter ((outfname)?outfname:infname), 
+		 eolstring);
+      }
+    }
+  }
 
   /*
    * update progress information
@@ -617,8 +971,8 @@ UUEncodePartial (FILE *outfile, FILE *infile,
 
   progress.action  = UUACT_ENCODING;
 
-  if ((res = UUEncodeStream (outfile, theifile,
-			     encoding, linperfile)) != UURET_OK) {
+  if ((res = UUEncodeStream (outfile, theifile, encoding, linperfile,
+			     crcptr, pcrcptr)) != UURET_OK) {
     if (infile==NULL) fclose (theifile);
     if (res != UURET_CANCEL) {
       UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
@@ -629,9 +983,11 @@ UUEncodePartial (FILE *outfile, FILE *infile,
     progress.action = 0;
     return res;
   }
+
   /*
    * print end line
    */
+
   if (feof (theifile) &&
       (encoding == UU_ENCODED || encoding == XX_ENCODED)) {
     fprintf (outfile, "%c%s",    
@@ -639,10 +995,30 @@ UUEncodePartial (FILE *outfile, FILE *infile,
 	     eolstring);
     fprintf (outfile, "end%s", eolstring);
   }
+  else if (encoding == YENC_ENCODED) {
+    if (numparts != 1) {
+      fprintf (outfile, "=yend size=%d part=%d pcrc32=%08lx",
+	       (partno*linperfile*128) < progress.totsize ? 
+	       linperfile*128 : (progress.totsize-(partno-1)*linperfile*128),
+	       partno,
+	       pcrc);
+    }
+    else {
+      fprintf (outfile, "=yend size=%d",
+	       progress.totsize);
+    }
+    if (feof (theifile))
+      fprintf (outfile, " crc32=%08lx", *crcptr);
+    fprintf (outfile, "%s", eolstring);
+  }
+
   /*
    * empty line at end does no harm
    */
-  fprintf (outfile, "%s", eolstring);
+
+  if (encoding != PT_ENCODED && encoding != QP_ENCODED) {
+    fprintf (outfile, "%s", eolstring);
+  }
 
   if (infile==NULL) {
     if (res != UURET_OK) {
@@ -678,11 +1054,14 @@ UUEncodeToStream (FILE *outfile, FILE *infile,
   FILE *theifile;
   int themode;
   int res;
+  crc32_t crc;
+  crc32_t *crcptr=NULL;
 
   if (outfile==NULL ||
       (infile == NULL&&infname==NULL) ||
       (outfname==NULL&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUEncodeToStream()");
     return UURET_ILLVAL;
@@ -719,7 +1098,7 @@ UUEncodeToStream (FILE *outfile, FILE *infile,
     theifile = infile;
   }
 
-  if (progress.fsize <= 0)
+  if (progress.fsize < 0)
     progress.fsize = -1;
 
   _FP_strncpy (progress.curfile, (outfname)?outfname:infname, 256);
@@ -736,7 +1115,23 @@ UUEncodeToStream (FILE *outfile, FILE *infile,
 	     UUFNameFilter ((outfname)?outfname:infname), 
 	     eolstring);
   }
-  if ((res = UUEncodeStream (outfile, theifile, encoding, 0)) != UURET_OK) {
+  else if (encoding == YENC_ENCODED) {
+    crc = crc32(0L, Z_NULL, 0);
+    crcptr = &crc;
+    if (progress.fsize == -1) {
+      fprintf (outfile, "=ybegin line=128 name=%s%s",
+	       UUFNameFilter ((outfname)?outfname:infname), 
+	       eolstring);
+    }
+    else {
+      fprintf (outfile, "=ybegin line=128 size=%ld name=%s%s",
+	       progress.fsize,
+	       UUFNameFilter ((outfname)?outfname:infname), 
+	       eolstring);
+    }
+  }
+
+  if ((res = UUEncodeStream (outfile, theifile, encoding, 0, crcptr, NULL)) != UURET_OK) {
     if (res != UURET_CANCEL) {
       UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_ERR_ENCODING),
@@ -746,15 +1141,31 @@ UUEncodeToStream (FILE *outfile, FILE *infile,
     progress.action = 0;
     return res;
   }
+
   if (encoding == UU_ENCODED || encoding == XX_ENCODED) {
     fprintf (outfile, "%c%s",    
 	     (encoding==UU_ENCODED) ? UUEncodeTable[0] : XXEncodeTable[0], 
 	     eolstring);
     fprintf (outfile, "end%s", eolstring);
   }
+  else if (encoding == YENC_ENCODED) {
+    if (progress.fsize == -1) {
+      fprintf (outfile, "=yend crc32=%08lx%s",
+	       crc,
+	       eolstring);
+    }
+    else {
+      fprintf (outfile, "=yend size=%ld crc32=%08lx%s",
+	       progress.fsize,
+	       crc,
+	       eolstring);
+    }
+  }
+
   /*
    * empty line at end does no harm
    */
+
   fprintf (outfile, "%s", eolstring);
 
   if (infile==NULL) fclose (theifile);
@@ -775,10 +1186,13 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
   char *oname=NULL, *optr, *ptr;
   FILE *theifile, *outfile;
   struct stat finfo;
+  crc32_t pcrc, crc;
+  crc32_t *pcrcptr=NULL, *crcptr=NULL;
 
   if ((diskname==NULL&&infname==NULL) ||
       (outfname==NULL&&infname==NULL) || (infile==NULL&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUEncodeToFile()");
     return UURET_ILLVAL;
@@ -890,7 +1304,7 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
 
   _FP_strncpy (progress.curfile, (outfname)?outfname:infname, 256);
 
-  progress.totsize  = (progress.totsize<=0) ? -1 : progress.totsize;
+  progress.totsize  = (progress.totsize<0) ? -1 : progress.totsize;
   progress.numparts = numparts;
 
   for (part=1; !feof (theifile); part++) {
@@ -946,19 +1360,22 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
       free (oname);
       return UURET_IOERR;
     }
-    fprintf (outfile, "%s", eolstring);
-    fprintf (outfile, "_=_ %s", eolstring);
-    if (numparts == -1)
-      fprintf (outfile, "_=_ Part %03d of file %s%s",
-	       part, UUFNameFilter ((outfname)?outfname:infname),
-	       eolstring);
-    else
-      fprintf (outfile, "_=_ Part %03d of %03d of file %s%s",
-	       part, numparts,
-	       UUFNameFilter ((outfname)?outfname:infname),
-	       eolstring);
-    fprintf (outfile, "_=_ %s", eolstring);
-    fprintf (outfile, "%s", eolstring);
+
+    if (encoding != YENC_ENCODED) {
+      fprintf (outfile, "%s", eolstring);
+      fprintf (outfile, "_=_ %s", eolstring);
+      if (numparts == -1)
+	fprintf (outfile, "_=_ Part %03d of file %s%s",
+		 part, UUFNameFilter ((outfname)?outfname:infname),
+		 eolstring);
+      else
+	fprintf (outfile, "_=_ Part %03d of %03d of file %s%s",
+		 part, numparts,
+		 UUFNameFilter ((outfname)?outfname:infname),
+		 eolstring);
+      fprintf (outfile, "_=_ %s", eolstring);
+      fprintf (outfile, "%s", eolstring);
+    }
 
     if (part==1 && (encoding == UU_ENCODED || encoding == XX_ENCODED)) {
       fprintf (outfile, "begin %o %s%s",
@@ -966,8 +1383,51 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
 	       UUFNameFilter ((outfname)?outfname:infname), 
 	       eolstring);
     }
+    else if (encoding == YENC_ENCODED) {
+      if (!crcptr) {
+        crc = crc32(0L, Z_NULL, 0);
+        crcptr = &crc;
+      }
+      pcrc = crc32(0L, Z_NULL, 0);
+      pcrcptr = &pcrc;
+      if (numparts != 1) {
+	if (progress.totsize == -1) {
+	  fprintf (outfile, "=ybegin part=%d line=128 name=%s%s",
+		   part,
+		   UUFNameFilter ((outfname)?outfname:infname), 
+		   eolstring);
+	}
+	else {
+	  fprintf (outfile, "=ybegin part=%d line=128 size=%ld name=%s%s",
+		   part,
+		   progress.totsize,
+		   UUFNameFilter ((outfname)?outfname:infname), 
+		   eolstring);
+	}
+
+	fprintf (outfile, "=ypart begin=%d end=%d%s",
+		 (part-1)*linperfile*128+1,
+		 (part*linperfile*128) < progress.totsize ? 
+		 (part*linperfile*128) : progress.totsize,
+		 eolstring);
+      }
+      else {
+	if (progress.totsize == -1) {
+	  fprintf (outfile, "=ybegin line=128 name=%s%s",
+		   UUFNameFilter ((outfname)?outfname:infname), 
+		   eolstring);
+	}
+	else {
+	  fprintf (outfile, "=ybegin line=128 size=%ld name=%s%s",
+		   progress.totsize,
+		   UUFNameFilter ((outfname)?outfname:infname), 
+		   eolstring);
+	}
+      }
+    }
+
     if ((res = UUEncodeStream (outfile, theifile,
-			       encoding, linperfile)) != UURET_OK) {
+			       encoding, linperfile, crcptr, pcrcptr)) != UURET_OK) {
       if (res != UURET_CANCEL) {
 	UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 		   uustring (S_ERR_ENCODING),
@@ -981,6 +1441,7 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
       _FP_free (oname);
       return res;
     }
+
     if (feof (theifile) &&
 	(encoding == UU_ENCODED || encoding == XX_ENCODED)) {
       fprintf (outfile, "%c%s",    
@@ -988,12 +1449,31 @@ UUEncodeToFile (FILE *infile, char *infname, int encoding,
 	       eolstring);
       fprintf (outfile, "end%s", eolstring);
     }
+    else if (encoding == YENC_ENCODED) {
+      if (numparts != 1) {
+	fprintf (outfile, "=yend size=%d part=%d pcrc32=%08lx",
+		 (part*linperfile*128) < progress.totsize ? 
+		 linperfile*128 : (progress.totsize-(part-1)*linperfile*128),
+		 part,
+		 pcrc);
+      }
+      else {
+	fprintf (outfile, "=yend size=%d",
+		 progress.totsize);
+      }
+      if (feof (theifile))
+	fprintf (outfile, " crc32=%08lx", crc); 
+      fprintf (outfile, "%s", eolstring);
+    }
+
     /*
      * empty line at end does no harm
      */
+
     fprintf (outfile, "%s", eolstring);
     fclose  (outfile);
   }
+
   if (infile==NULL) fclose (theifile);
   progress.action = 0;
   _FP_free (oname);
@@ -1012,6 +1492,22 @@ UUE_PrepSingle (FILE *outfile, FILE *infile,
 		char *outfname, int filemode,
 		char *destination, char *from,
 		char *subject, int isemail)
+{
+  return UUE_PrepSingleExt (outfile, infile,
+			    infname, encoding,
+			    outfname, filemode,
+			    destination, from,
+			    subject, NULL,
+			    isemail);
+}
+
+int UUEXPORT
+UUE_PrepSingleExt (FILE *outfile, FILE *infile,
+		   char *infname, int encoding,
+		   char *outfname, int filemode,
+		   char *destination, char *from,
+		   char *subject, char *replyto,
+		   int isemail)
 {
   mimemap *miter=mimetable;
   char *subline, *oname;
@@ -1019,7 +1515,8 @@ UUE_PrepSingle (FILE *outfile, FILE *infile,
   int res, len;
 
   if ((outfname==NULL&&infname==NULL) || (infile==NULL&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUE_PrepSingle()");
     return UURET_ILLVAL;
@@ -1036,18 +1533,28 @@ UUE_PrepSingle (FILE *outfile, FILE *infile,
   else
     mimetype = NULL;
 
+  if (mimetype == NULL && (encoding == PT_ENCODED || encoding == QP_ENCODED)) {
+    mimetype = "text/plain";
+  }
+
   if ((subline = (char *) malloc (len)) == NULL) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_OUT_OF_MEMORY), len);
     return UURET_NOMEM;
   }
 
-  if (subject)
-    sprintf (subline, "%s (001/001) - [ %s ]", subject, oname);
-  else
-    sprintf (subline, "[ %s ] (001/001)", oname);
-
-  fprintf (outfile, "Subject: %s%s", subline, eolstring);
+  if (encoding == YENC_ENCODED) {
+    if (subject)
+      sprintf (subline, "- %s - %s (001/001)", oname, subject);
+    else
+      sprintf (subline, "- %s - (001/001)", oname);
+  }
+  else {
+    if (subject)
+      sprintf (subline, "%s (001/001) - [ %s ]", subject, oname);
+    else
+      sprintf (subline, "[ %s ] (001/001)", oname);
+  }
 
   if (from) {
     fprintf (outfile, "From: %s%s", from, eolstring);
@@ -1057,13 +1564,23 @@ UUE_PrepSingle (FILE *outfile, FILE *infile,
 	     (isemail)?"To":"Newsgroups",
 	     destination, eolstring);
   }
-  fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
-  fprintf (outfile, "Content-Type: %s; name=\"%s\"%s",
-	   (mimetype)?mimetype:"Application/Octet-Stream",
-	   UUFNameFilter ((outfname)?outfname:infname),
-	   eolstring);
-  fprintf (outfile, "Content-Transfer-Encoding: %s%s",
-	   CTE_TYPE(encoding), eolstring);
+
+  fprintf (outfile, "Subject: %s%s", subline, eolstring);
+
+  if (replyto) {
+    fprintf (outfile, "Reply-To: %s%s", replyto, eolstring);
+  }
+
+  if (encoding != YENC_ENCODED) {
+    fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
+    fprintf (outfile, "Content-Type: %s; name=\"%s\"%s",
+	     (mimetype)?mimetype:"Application/Octet-Stream",
+	     UUFNameFilter ((outfname)?outfname:infname),
+	     eolstring);
+    fprintf (outfile, "Content-Transfer-Encoding: %s%s",
+	     CTE_TYPE(encoding), eolstring);
+  }
+
   fprintf (outfile, "%s", eolstring);
 
   res = UUEncodeToStream (outfile, infile, infname, encoding,
@@ -1080,6 +1597,24 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
 		 int partno, long linperfile, long filesize,
 		 char *destination, char *from, char *subject,
 		 int isemail)
+{
+  return UUE_PrepPartialExt (outfile, infile,
+			     infname, encoding,
+			     outfname, filemode,
+			     partno, linperfile, filesize,
+			     destination,
+			     from, subject, NULL,
+			     isemail);
+}
+
+int UUEXPORT
+UUE_PrepPartialExt (FILE *outfile, FILE *infile,
+		    char *infname, int encoding,
+		    char *outfname, int filemode,
+		    int partno, long linperfile, long filesize,
+		    char *destination,
+		    char *from, char *subject, char *replyto,
+		    int isemail)
 {
   static int numparts, themode;
   static char mimeid[64];
@@ -1088,9 +1623,12 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
   char *subline, *oname;
   long thesize;
   int res, len;
+  static crc32_t crc;
+  crc32_t *crcptr=NULL;
 
   if ((outfname==NULL&&infname==NULL) || (infile==NULL&&infname==NULL) ||
-      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED)) {
+      (encoding!=UU_ENCODED&&encoding!=XX_ENCODED&&encoding!=B64ENCODED&&
+       encoding!=PT_ENCODED&&encoding!=QP_ENCODED&&encoding!=YENC_ENCODED)) {
     UUMessage (uuencode_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_PARM_CHECK), "UUE_PrepPartial()");
     return UURET_ILLVAL;
@@ -1102,6 +1640,7 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
   /*
    * if first part, get information about the file
    */
+
   if (partno == 1) {
     if (infile==NULL) {
       if (stat (infname, &finfo) == -1) {
@@ -1132,7 +1671,7 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
 		     uustring (S_STAT_ONE_PART));
 	  numparts = 1;
 	  themode  = (filemode)?filemode:0644;
-	  thesize  = 0;
+	  thesize  = -1;
 	}
 	else {
 	  if (linperfile <= 0)
@@ -1157,20 +1696,22 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
       }
       theifile = infile;
     }
+
     /*
      * if there's one part only, don't use Message/Partial
      */
 
     if (numparts == 1) {
       if (infile==NULL) fclose (theifile);
-      return UUE_PrepSingle (outfile, infile, infname, encoding,
-			     outfname, filemode, destination,
-			     from, subject, isemail);
+      return UUE_PrepSingleExt (outfile, infile, infname, encoding,
+				outfname, filemode, destination,
+				from, subject, replyto, isemail);
     }
 
     /*
      * we also need a unique ID
      */
+
     sprintf (mimeid, "UUDV-%ld.%ld.%s",
 	     (long) time(NULL), thesize,
 	     (strlen(oname)>16)?"oops":oname);
@@ -1183,34 +1724,57 @@ UUE_PrepPartial (FILE *outfile, FILE *infile,
     return UURET_NOMEM;
   }
 
-  if (subject)
-    sprintf (subline, "%s (%03d/%03d) - [ %s ]", 
-	     subject, partno, numparts, oname);
-  else
-    sprintf (subline, "[ %s ] (%03d/%03d)",
-	     oname, partno, numparts);
 
-  fprintf (outfile, "Subject: %s%s", subline, eolstring);
+  if (encoding == YENC_ENCODED) {
+    if (partno == 1)
+      crc = crc32(0L, Z_NULL, 0);
+    crcptr = &crc;
+    if (subject)
+      sprintf (subline, "- %s - %s (%03d/%03d)", oname, subject,
+	       partno, numparts);
+    else
+      sprintf (subline, "- %s - (%03d/%03d)", oname,
+	       partno, numparts);
+  }
+  else {
+    if (subject)
+      sprintf (subline, "%s (%03d/%03d) - [ %s ]", 
+	       subject, partno, numparts, oname);
+    else
+      sprintf (subline, "[ %s ] (%03d/%03d)",
+	       oname, partno, numparts);
+  }
 
   if (from) {
     fprintf (outfile, "From: %s%s", from, eolstring);
   }
+
   if (destination) {
     fprintf (outfile, "%s: %s%s",
 	     (isemail)?"To":"Newsgroups",
 	     destination, eolstring);
   }
-  fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
-  fprintf (outfile, "Content-Type: Message/Partial; number=%d; total=%d;%s",
-	   partno, numparts, eolstring);
-  fprintf (outfile, "\tid=\"%s\"%s",
-	   mimeid, eolstring);
+
+  fprintf (outfile, "Subject: %s%s", subline, eolstring);
+
+  if (replyto) {
+    fprintf (outfile, "Reply-To: %s%s", replyto, eolstring);
+  }
+
+  if (encoding != YENC_ENCODED) {
+    fprintf (outfile, "MIME-Version: 1.0%s", eolstring);
+    fprintf (outfile, "Content-Type: Message/Partial; number=%d; total=%d;%s",
+	     partno, numparts, eolstring);
+    fprintf (outfile, "\tid=\"%s\"%s",
+	     mimeid, eolstring);
+  }
+    
   fprintf (outfile, "%s", eolstring);
 
   res = UUEncodePartial (outfile, theifile,
 			 infname, encoding,
 			 (outfname)?outfname:infname, NULL,
-			 themode, partno, linperfile);
+			 themode, partno, linperfile, crcptr);
 
   _FP_free (subline);
 
