@@ -81,7 +81,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.24 2002/03/11 09:15:47 fp Exp $";
+char * uulib_id = "$Id: uulib.c,v 1.28 2003/09/29 23:27:47 fp Exp $";
 
 #ifdef SYSTEM_WINDLL
 BOOL _export WINAPI 
@@ -242,6 +242,7 @@ static allomap toallocate[] = {
   { &uunconc_BHxlat,     256 * sizeof (int) },
   { &uunconc_save,     3*300 },  /* from uunconc.c:decoding buffer */
   { &uuscan_shlline,    1024 },  /* from uuscan.c:ScanHeaderLine() */
+  { &uuscan_shlline2,   1024 },  /* from uuscan.c:ScanHeaderLine() */
   { &uuscan_pvvalue,     300 },  /* from uuscan.c:ParseValue() */
   { &uuscan_phtext,      300 },  /* from uuscan.c:ParseHeader() */
   { &uuscan_sdline,      300 },  /* from uuscan.c:ScanData() */
@@ -654,6 +655,12 @@ UUFNameFilter (char *fname)
 
 int UUEXPORT
 UULoadFile (char *filename, char *fileid, int delflag)
+{
+  return UULoadFileWithPartNo(filename, fileid, delflag, -1);
+}
+
+int UUEXPORT
+UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
 {
   int res, sr, count=0;
   struct stat finfo;
@@ -748,6 +755,9 @@ UULoadFile (char *filename, char *fileid, int delflag)
       return UURET_IOERR;
     }
 
+    if (partno != -1)
+      loaded->partno = partno;
+
     if ((loaded->uudet == QP_ENCODED || loaded->uudet == PT_ENCODED) &&
 	(loaded->filename == NULL || *(loaded->filename) == '\0') &&
 	!uu_handletext && (loaded->flags&FL_PARTIAL)==0) {
@@ -829,6 +839,15 @@ UULoadFile (char *filename, char *fileid, int delflag)
     if (loaded->uudet)
       count++;
   }
+  if (ferror (datei)) {
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_READ_ERROR), filename,
+	       strerror (uu_errno = errno));
+    UUCheckGlobalList ();
+    progress.action = 0;
+    fclose (datei);
+    return UURET_IOERR;
+  }
   fclose (datei);
 
   if (!uu_fast_scanning && count==0) {
@@ -1004,8 +1023,14 @@ UUDecodeFile (uulist *thefile, char *destname)
     }
   }
 
-  fclose (target);
   fclose (source);
+  if (fclose (target)) {
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_WR_ERR_TARGET),
+	       uugen_fnbuffer, strerror (uu_errno = errno));
+    unlink (uugen_fnbuffer);
+    return UURET_IOERR;
+  }
 
   /*
    * after a successful decoding run, we delete the temporary file
